"""
# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

HDF basic writer class PR by David Grote, LLNL
Heavily modified from PR.py originally written by Paul Dubois, LLNL, to use
HDF files.
$Id: PRpyt.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $

This provides a nice wrapper over the pytables for reading general
self-describing data sets written out with PWpyt.PW.

"""

import pprint
import tables
import sys
import re
import cPickle
import string

_version = '0.4'

class PRError(Exception):
    pass

class PR:
    """HDF file read-access class.
Access to data in the file is available via attributes. For example, to get
the value of x from a file, do the following...

ff = PR(filename)
x = ff.x

There is also the read method
x = ff.read('x')
which is useful for names that are not usable as python attributes.
    """
    file_type = "HDF"

    """
Extra documentation:

The _cache is a dictionary for convenience. When a name is first looked up,
it is added to the cache so the next time, that value in the cache is used
instead of having to read in the data from the file again. This is meant to
save time. When the file is opened, all of the variables in the _ints and
_floats are added to the cache, as well as everything that was in the pickle
dict.  

The _names is a list of all variables in the top level of the file, i.e. 
group='/'. This includes the names in the _ints and _floats tables, the names
in the pickle dict, and all of the other names that were directly written out.

When the file is opened, a scan is done looking for hashed names, names that
were too long to include directly in the _ints and _floats tables. The
translation to the original name is stored in _hashednames. It is only used
when the tables are read into the cache.

The delimiter can be used to write complicated names and is used heavily by
the Forthon package. pytables issues complaints if '@' is used, so an option
is given in PW to specify it. PW writes the delimiter is uses to the file so
that PR can read it in.

    """

    def __del__(self):
        self.close()

    def __getattr__(self, name):
        # --- Read in the variable from the file (or get it from the cache).
        return self.read(name)

    def __init__(self, filename, group='/', verbose = 1):
        """PR(filename='', group='/',verbose=1) opens filename"""

        self._file = None
        self._cache = {}
        self._names = []
        self._hashednames = {}
        self.set_verbosity(verbose)
        self.set_group(group)
        self._fixdelimiter = re.compile('@')
        if filename:
            self.open(filename, group,mode='r')

    def __repr__(self):
        if self.is_open():
            current_mode = 'opened for reading'
        else:
            current_mode = '(PR object not open on any file)'
        return 'HDF file %s %s.' % (self.inquire_filename(), current_mode)

    __str__ = __repr__

    def check_open(self):
        "check_open(): raise exception if not open for read."
        if not self.is_open():
            raise PRError, 'PR object not open for read.'

    def close(self):
        "close(): close the file."
        if self.is_open():
            if self.inquire_verbosity():
                print "Closing HDF file ",self.inquire_filename()
            self._file.close()

            self._file = None
            self._cache = {}
            self._names = []
            self._hashednames = {}

    def inquire_filename(self):
        "inquire_filename() = name of this file."
        if self.is_open ():
            return self._file.filename
        else:
            return ''

    def inquire_ls(self,group=None):
        """inquire_ls(group=None) = list of those names 
        in the file which represent objects in the given group.
        The default groups gets all."""
        self.check_open()
        if group is None or group == '/':
          # --- Return the list of all names, including names from the ints
          # --- and floats tables, and from the pickledict.
          return self._names
        else:
          # --- Get only names in the specified group
          ll = self._file.listNodes(group,classname='Leaf')
          ll = map(lambda l:l.name,ll)
          # --- Fix the delimiter in all of the names if needed.
          if self._delimiter != '@':
              ll = map(lambda l:self._fixdelimiter.sub(self._delimiter,l),ll)
          return ll

    def inquire_mode(self):
        "inquire_mode() = mode ('r', 'w', or 'a') of this file."
        self.check_open()
        return self._file.mode

    def inquire_names(self):
        "inquire_names() = sorted list of registered names"
        return self.inquire_ls()

    def inquire_group(self):
        "inquire_group() = present HDF group"
        return self._group

    def inquire_verbosity(self):
        "inquire_verbosity() = current value of verbose flag."
        return self._verbose

    def is_open(self):
        "is_open() = true if file is open for read"
        return not (self._file == None)

    def print_names(self, file=sys.stdout):
        """print_list(file=sys.stdout): pretty print the list of
        registered names to file."""
        pprint.pprint(self.inquire_names(), file)

    def open(self, filename, group=None,mode='r'):
        "open(name, group=None,mode='r'): open file, optionally starting at group"
        self.close()
        if filename:
            if group is None: group = self.inquire_group()
            self._file = tables.openFile(filename,mode=mode,rootUEP=group)
            self.set_group(group)
            self._read_delimiter()
            self._scan_Nodes()
            self._load_cache_with_pickledict()
            self._ints = self._file.getNode(group,'_ints')
            self._readtable(self._ints)
            self._floats = self._file.getNode(group,'_floats')
            self._readtable(self._floats)

    def read(self,name):
        "read(name) = the value of name as a Python object."
        self.check_open()
        try:
            return self._cache[name]
        except KeyError:
            # --- Replace the delimiter in the input name with the one that
            # --- was used when writing the file.
            if self._delimiter != '@':
                name = self._fixdelimiter.sub(self._delimiter,name)

            # --- Read in the data.
            node = self._file.getNode(self.inquire_group(),name)
            data = node.read()

            # --- Handle special cases
            if node.title == "Pickled":
              data = cPickle.loads(data)
            if node.title == 'ZeroLength':
              data = numpy.zeros(data[:-1],dtype=string.ascii_letters[data[-1]])

            # --- Add the data to the cache
            self._cache[name] = data

            # --- And finally return it.
            return data

    def _read_delimiter(self):
        "read in the delimiter"
        node = self._file.getNode(self.inquire_group(),'_delimiter_')
        if node.title != 'Delimiter':
            raise PRError,"Error: the delimiter was overwritten"
        self._delimiter = node.read()

    def _scan_Nodes(self):
        self.check_open()
        group = self.inquire_group()
        for node in self._file.iterNodes(group,classname='Leaf'):
            
            if node.title == 'HashedName':
                # --- Find hashed names in the tables
                hashedname = node.name
                origname = node.read()
                self._hashednames[hashedname] = origname
            elif node.name in ['_ints','_floats','_pickledict']:
                # --- These names are special
                pass
            else:
                # --- Save the rest of the names in the names list
                # --- Fix the delimiter in all of the names if needed.
                if self._delimiter != '@':
                    name = self._fixdelimiter.sub(self._delimiter,name)
                self._names.append(node.name)

    def _load_cache_with_pickledict(self):
        # --- Read in the pickled dictionary
        node = self._file.getNode(self.inquire_group()+'_pickledict')
        if node.title != 'PickleDict':
            print "Warning: the pickle dictionary was written over, the data may not be reliable"
        globalpickledict = cPickle.loads(node.read())

        # --- Add the unpickled data to the cache
        self._cache.update(globalpickledict)

        # --- Add all of the names to the list
        self._names += globalpickledict.keys()

    def _readtable(self,table):
        for i in table.iterrows():
            name = i['name']
            try: name = self._hashednames[name]
            except KeyError: pass
            self._cache[name] = i['value']
            self._names.append(name)

    def set_group(self, name):
        """set_group(name)
        -- change HDF group to name, return status"""
        if name[0] == '/':
            group = name
        elif len(self.inquire_group()) == 1:
            group = '/' + name
        else:
            group = self.inquire_group() + '/' + name
        if group[-1] != '/': group = group + '/'
        self._group = group

    def set_verbosity(self, flag):
        """verbose(flag) sets verbosity level to flag.
        0 for quiet operation, 
        1 to report closing only, 
        2 to report access to data."""
        if 0 <= flag <= 2:
            self._verbose = flag
        else:
            raise PRError, 'Illegal value for verbosity: '+`flag`




warp_version = "$Id: uedge.py,v 7.1 2018/10/18 18:13:14 meyer8 Exp $"
# import all of the neccesary packages
import __main__
import sys

#test for numpy
try:
  import numpy
  __main__.__dict__['with_numpy'] = 1
except:
  try:
    import Numeric
  except:
    raise SystemExit, "Neither numeric nor numpy present"

try:
  with_numpy = __main__.__dict__['with_numpy']
except KeyError:
  with_numpy = ('--with-numpy' in sys.argv)
  __main__.__dict__['with_numpy'] = with_numpy

if with_numpy:
  from numpy import *
  ArrayType = ndarray
  def gettypecode(x):
    return x.dtype.char
  def oldnonzero(a):
    return a.nonzero()[0]
else:
  from Numeric import *
  import MA
  def gettypecode(x):
    return x.typecode()
  oldnonzero = nonzero
import os.path
import time

if not with_numpy:
   #-- Set this to a more reasonable value
   MA.set_print_limit(10000)

# --- Since gist is only loaded on PE0 in the parallel
# --- environment, the parallel module must also be loaded in to check
# --- whether this is in fact running in parallel.

# --- This creates a logical lparallel which is true when running in parallel
# --- and false when running serial.
# --- This also creates a number of routines which are needed for parallel
# --- data handling. These routines all have sensible default results when
# --- not running in parallel.
from ueparallel import *

try:
  if me == 0 and sys.platform != 'mac':
    from gist import *
  else:
    from gistdummy import *
except ImportError:
  me = 0

# Print a greeting
# mmiah: Actually, lets not.
# print "Welcome to PyUedge"
# Import the uedgeC shared object which contains all of UEDGE
# from Forthon import *
import Forthon
try:
  import PyPDB
  from PyPDB import PW,PR
  from PyPDB.pypdb import *
except:
  # print "Unable to import PyPDB or * from PyPDB.pypdb."
  # print "Will proceed to try to import pypdb in case of old installation."
  try:
    from pypdb import *
  except:
    # print "pypdb not found."
    pass
#    raise
#  mmiah: Why are we reraising the exception?  We don't
#         necessarily need PyPDB
from uedgeC import *
from uedgeutils import *

# --- The UEDGE modules must be imported in the order below because of
# --- linking dependencies.
from compy import *
from aphpy import *
from apipy import *
from bbbpy import *
from flxpy import *
from grdpy import *
from svrpy import *
from wdfpy import *

# print "Packages loaded: ", Forthon.package()
# --- Add stuff to the path
import sys

# --- Set default runid to first filename in the command line, stripping off
# --- the .py suffix.
if sys.argv[0]:
  if sys.argv[0][-3:] == '.py':
    h,t = os.path.split(sys.argv[0][:-3])
    runid = t
    del h,t
  else:
    h,t = os.path.split(sys.argv[0])
    runid = t
    del h,t

# --- Check if the compiler was ifort - if so, set the stacksize unlimited
# --- The fcompname is not yet be available yet if Forthon is not up to date
try:
  if fcompname == 'ifort':
    import resource
    resource.setrlimit(resource.RLIMIT_STACK,(-1,-1))
except:
  pass
##############################################################################
######  Don't put anything below this line!!! ################################
##############################################################################




import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.axes as ax
from matplotlib.collections import PolyCollection
from matplotlib.colors import LinearSegmentedColormap
from matplotlib import rcdefaults
from matplotlib import interactive
from uedge import *

# This file defines a function to plot the UEDGE mesh, and
# then calls the function to plot the entire mesh.
# To use this file in a simple way, give the following ands:
#    read plotmesh
#    nf
# The function could then be used in a more sophisticated way
# to plot portions of the mesh, possibly with customized plot limits
# (by resetting acom.ny of r_min, r_max, z_min, and z_max):
#    call plotmesh(ixmn,ixmx,iymn,iymx)
#    nf
# where ixmn, ixmx, iymn, and iymx are integer variables or
# expressions.  Always give an "nf" and after reading the file
# plotmesh or calling the function plotmesh.

# DEFINE THE PLOT FUNCTION --

def plotmesh(ixmin=None,ixmax=None,iymin=None,iymax=None,
          r_min=None,r_max=None,z_min=None,z_max=None,title=None,
          block=True):
   """
   plotmesh(ixmin=<int>,ixmax=<int>,iymin=<int>,iymax=<int>
            title=<string>,r_min=<val>,r_max=<val>,z_min=<val>,z_max=<val>,
            block=<True|False>)
      where ixmin, ixmax, iymin, and iymax are integer variables or
      expressions used to plot a portion of the grid. title is used as
      both the title and the figure name. Block default is True.
      
      The plot axis limits may be specified with r_rmin,r_max,z_min,z_max.
   """

   if ixmin == None: ixmin = com.nxomit
   if ixmax == None: ixmax = (com.nxm-1)
   if iymin == None: iymin = 0
   if iymax == None: iymax = (com.ny-1)
   if r_min == None: r_min = com.rm.min()
   if r_max == None: r_max = com.rm.max()
   if z_min == None: z_min = com.zm.min()
   if z_max == None: z_max = com.zm.max()

   rcdefaults()
   if title == None:
    title='Uedge Grid'
   plt.figure(title,figsize=(4.0,8.0))
   plt.title(title)

   try:
      plt.plot(com.xlim,com.ylim,'k-',label='Limiter',linewidth=3)
      plt.plot(com.xlim,com.ylim,'y-',label='Limiter',linewidth=1)
      plt.plot(com.rbdry,com.zbdry,'b-',label='Last Closed')
   except:
      pass
   plt.ylim(z_min,z_max)
   plt.xlim(r_min,r_max)
   for ix in range(ixmax-ixmin+1):
       for iy in range(iymax-iymin+1):
           r0 = [com.rm[ix,iy,1],com.rm[ix,iy,2],com.rm[ix,iy,4],com.rm[ix,iy,3],com.rm[ix,iy,1]]
           z0 = [com.zm[ix,iy,1],com.zm[ix,iy,2],com.zm[ix,iy,4],com.zm[ix,iy,3],com.zm[ix,iy,1]]
           plt.plot(r0,z0,'k-',label='Grid',linewidth=1)
   plt.ylabel('Z (m)')
   plt.xlabel('R (m)')
   plt.axes().set_aspect('equal')



   
   #plt.show(block=block)
   plt.ion()
   plt.show()
   plt.pause(0.001)


def plotmeshval(val,ixmin=None,ixmax=None,iymin=None,iymax=None,
          r_min=None,r_max=None,z_min=None,z_max=None,title=None,units=None,
          block=True):
   """
   plotmeshval(val,ixmin=<int>,ixmax=<int>,iymin=<int>,iymax=<int>
            title=<string>,units=<string>,block=<True|False>)
      Display 2-D quantity using polyfill.
      where ixmin, ixmax, iymin, and iymax are integer variables or
      expressions used to plot a portion of the grid. title is used as
      both the title and the figure name. Units are displayed in the
      side colorbar. Block default is True.

      The plot axis limits may be specified with r_rmin,r_max,z_min,z_max.
   """
   if ixmin == None: ixmin = com.nxomit
   if ixmax == None: ixmax = (com.nxm-1)
   if iymin == None: iymin = 0
   if iymax == None: iymax = (com.ny-1)
   if r_min == None: r_min = com.rm.min()
   if r_max == None: r_max = com.rm.max()
   if z_min == None: z_min = com.zm.min()
   if z_max == None: z_max = com.zm.max()
   rcdefaults()
   if title == None:
    title='Uedge'
   fig, ax = plt.subplots()

   verts = np.array([])
   z = np.array([])
   for ix in range(ixmax-ixmin+1):
       for iy in range(iymax-iymin+1):
           v = []
           v.append([com.rm[ix,iy,1],com.zm[ix,iy,1]])
           v.append([com.rm[ix,iy,2],com.zm[ix,iy,2]])
           v.append([com.rm[ix,iy,4],com.zm[ix,iy,4]])
           v.append([com.rm[ix,iy,3],com.zm[ix,iy,3]])
           verts = np.append(verts,v)
           z = np.append(z,val[ix,iy])
   verts = verts.reshape(len(z),4,2)
   ax.set_title(title)
   ax.set_ylabel('Z (m)')
   ax.set_xlabel('R (m)')
   ax.set_aspect('equal')

   coll = PolyCollection(verts,array=z,cmap=cm.jet,edgecolors='face')
   ax.add_collection(coll)
   ax.autoscale_view()
   cbar = fig.colorbar(coll,ax=ax)
   #if units != None: cbar.ax.set_ylabel(units,rotation=-90,va='bottom')
   if units != None: cbar.ax.set_ylabel(units,va='bottom')
   plt.ylim(z_min,z_max)
   plt.xlim(r_min,r_max)

   #plt.show(block=block)
   plt.ion()
   plt.show()
   plt.pause(0.001)


def profile(rval,zval,title=None,style=None,linewidth=None,xlabel=None,ylabel=None):
   """
   profile(rval,zval,title=<None>)
      title is used as both the title and the figure name. 
      Interactive is turned on so subsequent calls go to the same plot
      Style encoded color, line, and marker.  See matplotlib documention.
      examples: black solid line  - style='k-'
                red circle marks  - style='ro'
                green x marks and dotted line - style='gx--'
      
   """

   rcdefaults()
   interactive(True)
   if title == None:
      title='Uedge Profile'
   if style == None:
      style = 'k-'
   if linewidth == None:
      lw = 1
     
   plt.figure(title,figsize=(4.0,8.0))
   plt.title(title)

   try:
      plt.plot(rval,zval,style,linewidth=lw)
   except:
      pass
   if ylabel != None:
      plt.ylabel(ylabel)
   if xlabel != None:
      plt.xlabel(xlabel)
   #plt.axes().set_aspect('equal')
   plt.ion()
   plt.show()
   plt.pause(0.001)






# Import all required libraries

import numpy
import pylab
import string

#
# The purpose of this class is to read a gridue file and present the
# data in a format friendly for use elsewhere
#
class UeReadGrid:

#
# Constructor
#
    def __init__(self):
        pass

    def readData(self, fname):
        fh = open(fname, 'r')

        lns = fh.readlines()

        # Read the header information including metadata and grid shape
        ln1 = lns.pop(0).split()

        self.xxr = ln1[0]
        self.yyr = ln1[1]

        self.xsp1 = ln1[2]
        self.xsp2 = ln1[3]
        self.ysp = ln1[4]

        lns.pop(0)

        # Reshape the grid data to be linear
        data = numpy.zeros( (lns.__len__() * 3), numpy.float )
        print data.shape
        for i in range(0, lns.__len__()-1):
            ll = lns[i].split()
            print ll
            data[3*i  ] = string.atof( ll[0] )
            data[3*i+1] = string.atof( ll[1] )
            data[3*i+2] = string.atof( ll[2] )

        rml = 0
        rmh = (self.xxr+2) * (self.yyr+2) * 5
        self.rm = data[rml:rmh].reshape( (self.xxr+2, self.yyr+2, 5) )

        zml = rmh
        zmh = rmh + (self.xxr+2) * (self.yyr+2) * 5
        self.zm = data[zml:zmh].reshape( (self.xxr+2, self.yyr+2, 5) )

uegrid = UeReadGrid()
uegrid.readData('gridue')
pylab.plot(uegrid.rm[:,:,0], uegrid.zm[:,:,0], 'b+')



#!/usr/bin/env python
#
# $Id: convert1.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $
#
# To try solving linear critical gradient

import sys
import os
import getopt
import string
import convert
from convert import *

# define the mppl to f90 class
class M2F(generic):
  suffixin = "m"
  suffixout = "F"
  subrules = globalsubrules + M2Fsubrules

def usage():
  print "Usage: convert1.py -i <indir> -o <outdir> <infile>"

r"""
main(argv: array of strings)
"""
def main(argv):
  try:
    opts, args = getopt.getopt(sys.argv[1:], "hi:o:", ["help",
	"indir=", "outdir="])
  except getopt.GetoptError:
# print help information and exit:
    usage()
    sys.exit(2)

  indir = "."
  outdir = "."
# Go through args
  for o, a in opts:
    if o in ("-h", "--help"):
      usage()
      sys.exit()
    if o in ("-i", "--indir"):
      indir = a
    elif o in ("-o", "--outdir"):
      outdir = a
  # print "args =", args

# Extract the file
  fn = args[0]
  # print "Converting " + fn

# Do the conversion
  convert.M2F = M2F
  m2f = M2F(indir, outdir)
  # m2f.outdir = outdir
  m2f.processfile(fn)

  return 0

if __name__ == '__main__':
  sys.exit(main(sys.argv))




# Setup file to run time-dependently using dtreal
# Change dtreal for starting dt and savefname to change pfb file name
# Once variables are set, read rdrundt to execute a time-dependent run

#integer ii1,ii2,irev,ii1max,ii2max,initjac
#real8 dtreal_sav, dt_tot, mult_dt, ftol_dt, t_stop, dt_max, dtmfnk3
#real8 tstor_s, tstor_e, dt_stor, deldt_min, ftol_min, rdtphidtr
#integer n_stor, i_stor, ipt, ismfnkauto, nfe_tot
#character*5 savefname
i_stor = 0
dt_tot = 0.
nfe_tot = 0
savefname = "it333" 	# change these 5 charac. for pfb name pfdt_"savefname"
                        # also creates pftstor_... for time-dependent arrays
##dtreal = 1.e-9		# starting value of dtreal
rdtphidtr = 1e20	# ratio dtphi/dtreal
ismfnkauto = 1		# if =1, mfnksol=3 for dtreal<dtmfnk3, otherwise=-3
dtmfnk3 = 1e-4		# dtreal for mfnksol sign change if ismfnkauto=1
mult_dt = 3.4		# factor expanding dtreal after ii2max steps
ii1max = 100		# number of changes to dtreal
ii2max = 5		# number of timesteps at current dtreal
ftol_dt = 1.e-4		# fnrm tolerance for the time-dependent steps
ftol_min = 1e-9		# value of fnrm where time advance will stop
dt_tot = 0.		# total time accumulated for run (output, not input)
t_stop = 100.		# value of dt_tot (sec) where calculation will stop
dt_max = 100.		# maximum time step for dtreal
deldt_min = 0.04	# minimum relative change allowed for model_dt > 0
initjac = 0		# if=1, calc initial Jac upon reading rdcontdt
###ismmaxuc = 1		# =1 for internal calc mmaxu; =0,set mmaxu & don't chng
irev = -1		# flag to allow reduced dt advance after cutback
bbb.rlx = 0.9		# max. change in variable at each linear iteration
bbb.itermx = 10		# max. number of linear iterations allowed
tstor_s = 1e-3		# beginning time for storing solution
tstor_e = 4e-2		# ending time for storing solution
n_stor = 0		# number of linearly spaced storage points
ipt = 1			# index of one variable whose value printed at each out
			# if ipt not reset from unity, ipt=idxte(nx,iysptrx+1)




"""
# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See LEGAL.LLNL for full text and disclaimer.

HDF basic writer class PW by David Grote, LLNL
Heavily modified from PW.py originally written by Paul Dubois, LLNL, to use
HDF files.
$Id: PWpyt.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $

This writes all scalars into two tables, one for ints and one for floats.
Everything else is written out using createArray.
Note that writing out scalars to tables is faster than writing then out
individually using createArray.
For scalars, the setup requires a maximum name length. Any names longer are
hashed.
Arrays are written out directly. Everything else is put into a dictionary
which is written out on close.

"""
import tables
from tables.exceptions import NaturalNameWarning
import cPickle
import re
import string
import numpy
import warnings
import types

_version = '0.4'

class PWError(Exception):
    pass

_max_name_len = 200
_transtable = (10*string.ascii_lowercase)[:256]

class IntScalar(tables.IsDescription):
    name = tables.StringCol(_max_name_len)
    value = tables.IntCol()
class FloatScalar(tables.IsDescription):
    name = tables.StringCol(_max_name_len)
    value = tables.FloatCol()

class PW:
    "HDF file writer class."
    file_type = "HDF"

    def __del__(self):
        "Close any file open when this object disappears."
        self.close()

    def __init__(self, filename=None, mode="w", group='/',
                 verbose = 1, delimiter='@'):
        "PW(filename='', verbose=1) creates filename if given" 
        self.__dict__['_delimiter'] = delimiter
        self.__dict__['_file'] = None
        self.set_verbosity(verbose)
        self.set_group(group)
        if filename is not None:
            self.open(filename,group,mode)

        self.__dict__['_globalpickledict'] = {}

        # --- This allows replacement of '@' with a user specified
        # --- delimiter.
        self.__dict__['_fixdelimiter'] = re.compile('@')

        # --- Turn off the NaturalNameWarning in this case since the
        # --- delimiter cannot be used in a python name.
        if self._delimiter == '@':
            warnings.simplefilter('ignore',NaturalNameWarning)

    def __setattr__(self, name, value):
        self.write(name, value)

    def __repr__(self):
        if self.is_open():
            current_mode = 'opened for writing'
            return 'HDF file %s %s.' % \
               (self.inquire_filename(), current_mode)
        else:
            return PWError,'(PW object not open on any file)'

    __str__ = __repr__

    def check_open(self):
        "check_open(): raise exception if not open for write."
        if not self.is_open():
            raise PWError, 'PW object not open for write.'

    def close(self):
        "close(): close the file."
        h = self.inquire_file()
        if h is not None:
            if self.inquire_verbosity():
                print "Closing HDF file being written:",self.inquire_filename()

            # --- Write out the list of things that have to be pickled,
            # --- pickling them all at once.
            q = cPickle.dumps(self._globalpickledict,-1)
            h.createArray(self.inquire_group(),'_pickledict',q,
                          title='PickleDict')

            # --- Write out the delimiter used.
            h.createArray(self.inquire_group(),
                          '_delimiter_',self._delimiter,
                          title='Delimiter')

            h.flush()
            h.close()

        # --- Reset various quantities
        self.__dict__['_file'] = None
        self.__dict__['_ints'] = None
        self.__dict__['_floats'] = None
        self.__dict__['_globalpickledict'] = {}

        if self._delimiter == '@':
            warnings.simplefilter('default',NaturalNameWarning)

    def inquire_filename(self):
        "inquire_filename() = name of this file."
        if self.is_open():
            return self.inquire_file().filename
        else:
            return ''

    def inquire_file(self):
        "inquire_file() = object open on this file."
        return self._file

    def inquire_mode(self):
        "inquire_mode() = mode('w', or 'a') of this file."
        self.check_open()
        return self._mode

    def inquire_group(self):
        "inquire_group() = present HDF group"
        return self.__dict__['_group']

    def inquire_verbosity(self):
        "inquire_verbosity() = current value of verbose flag."
        return self._verbose_flag

    def is_open(self):
        "is_open() = true if file is open"
        if self.inquire_file() is None: return 0
        return self.inquire_file().isopen

    def open(self, filename,group=None,mode = "w"):
        "open(filename, 'w')"
        self.close()
        assert mode in ['w','a'],Exception("Improper mode: " + mode)
        if group is None: group = self.inquire_group()
        self.set_group(group)
        self.__dict__['_mode'] = mode
        self.__dict__['_file'] = tables.openFile(filename,mode=mode,
                                                 rootUEP=self.inquire_group())
        if mode == 'a':
            # --- If mode is append, check if the ints and floats tables
            # --- have been created. If not, catch the error.
            try:
                self.__dict__['_ints'] = self.inquire_file().root._ints
                self.__dict__['_floats'] = self.inquire_file().root._floats
            except LookupError:
                pass

        # --- If mode is 'w' or if the mode is 'a' but the tables have
        # --- not yet been written, then create them.
        if self.__dict__['_ints'] is None:
            self.__dict__['_ints'] = self.inquire_file().createTable(
                                          self.inquire_group(),
                                          '_ints',IntScalar,"Scalar Ints")
        if self.__dict__['_floats'] is None:
            self.__dict__['_floats'] = self.inquire_file().createTable(
                                          self.inquire_group(),
                                          '_floats',FloatScalar,"Scalar Floats")

    def make_group(self, name, title=''):
        """make_group(name) 
        -- create a new HDF group, return status"""
        self.check_open()
        self.inquire_file().createGroup(self.inquire_group(),name,title=title)
        return 1

    def set_group(self, name):
        """set_group(name) 
        -- change HDF group to name, return status"""
        if name[0] == '/':
            group = name
        elif len(self.inquire_group()) == 1:
            group = '/' + name
        else:
            group = self.inquire_group() + '/' + name
        self.__dict__['_group'] = group

    def set_verbosity(self, flag):
        """set_verbosity(flag) sets verbosity level to flag.
        0 for quiet operation, 
        1 to report closing only, 
        2 to report access to data."""
        if 0 <= flag <= 2:
            self.__dict__['_verbose_flag'] = flag
        else:
            self.__dict__['_verbose_flag'] = 2

    def fixlongnames(self,name):
      """
Handle names longer than the maximum length allowed by IntScalar and
FloatScalar.
      """
      origname = name
      if len(name) > _max_name_len:
          hash = string.translate(md5.new(origname).digest(),_transtable)
          hashedname = hash[:_max_name_len]
          h.createArray(self.inquire_group(),
                        hashedname,
                        origname,
                        title='HashedName')
      else:
          hashedname = name
      return origname,hashedname

    def write(self, name, quantity, title=''):
        """Write quantity to file as 'name'"""
        self.check_open()
        if self.inquire_verbosity() > 1: print "PW::write writing", name
        h = self.inquire_file()

        if self._delimiter != '@':
            name = self._fixdelimiter.sub(self._delimiter,name)

        if isinstance(quantity,types.IntType):
            # --- integers are put into the ints table
            origname,name = self.fixlongnames(name)
            self._ints.row['name'] = name
            self._ints.row['value'] = quantity
            self._ints.row.append()

        elif isinstance(quantity,types.FloatType):
            # --- floats are put into the floats table
            origname,name = self.fixlongnames(name)
            self._floats.row['name'] = name
            self._floats.row['value'] = quantity
            self._floats.row.append()

        elif isinstance(quantity,numpy.ndarray):
            # --- Arrays are written out directly
            if min(quantity.shape) == 0:
                # --- Arrays with a dimension of length zero cannot be
                # --- direcrly written out. Instead, write out its shape
                # --- and type so it can be later reconstructed.
                arrayinfo = (list(quantity.shape) +
                             [string.ascii_letters.find(quantity.dtype.char)])
                h.createArray(self.inquire_group(),name,arrayinfo,"ZeroLength")
            else:
                h.createArray(self.inquire_group(),name,quantity,title=title)

        else:
            # --- Everything else is either skipped or put into the global
            # --- pickle dict

            try:
                try:
                    if quantity.__class__.__module__ == '__main__':
                        if self.inquire_verbosity():
                            print name,' is being skipped since it is an instance of a class defined in main and therefore cannot be unpickled'
                        return
                except:
                    pass
                try:
                    if quantity.__module__ == '__main__':
                        if self.inquire_verbosity():
                            print name,' is being skipped since it is a class defined in main and therefore cannot be unpickled'
                        return
                except:
                    pass
                # --- Check if an object can be pickled. This will be inefficient
                # --- for large objects since they will be pickled twice, but this
                # --- is the only safe way.
                try:
                    q = cPickle.dumps(quantity,-1)
                    del q
                except:
                    if self.inquire_verbosity():
                      print name,' is being skipped since it could not be written or pickled'
                    return
                # --- Things that need to be pickled will all be written out at
                # --- once in the same pickle when the file is being closed so that
                # --- multiple references to the same object are handled correctly.
                self._globalpickledict[name] = quantity
                return
            except:
                pass

            raise PWError,"Could not write the variable %s"%name

if __name__ == "__main__":
    f=PW("foo.hdf")
    a = 1
    b = 2.0
    c = "Hello world"
    from multiarray import *
    d = array([1.,2., 3.])
    e = array([1,2,3])
    g = array(["hello", "world", "array"])
    h = array([[1.,2.,3.], [4.,5.,6]])
    k = 3
    f.a = a
    f.b = b
    f.c = c
    f.d = d
    f.e = e
    f.g = g
    f.h = h
    f.close()
    f.open("foo.hdf", "a")
    f.k = k
    f.close()
# read-back test
    from PRpyt import PR
    f = PR('foo.hdf')
    for x in f.inquire_names():
        print x, "is", eval(x), ", in file it is", eval('f.'+x)
    f.close()
# record-writing
    g = PW('goo.hdf')
    g.set_verbosity(1)
    xh = array([0.]*4)
    for i in range(len(xh)):
        x = i / 10.
        g.write('xh', x, i + 1)
        xh [i] = x
    g.close()
    g = PR('goo.hdf')
    print "xh is", xh, ", file it is ", g.xh
    g.close()









from uedge import *
com.nxleg=2*com.nxleg
com.nxcore=2*com.nxcore
com.nycore=2*com.nycore
com.nysol=2*com.nysol
if com.nxomit > 0:
  if com.geometry=="dnbot":
    com.nxomit = com.nxleg[0,0]+com.nxcore[0,0] + 2*com.nxxptxx + 1
  else:
    com.nxomit=2*(com.nxomit-2*com.nxxptxx) + 2*com.nxxptxx  # assumes com.nxomit removes 1/2 SOL
if com.nyomitmx == 1: com.nysol = 1
if grd.kxmesh == 4:
   grd.dxgasold=grd.dxgas
   grd.alfxold=grd.alfx
   grd.alfx=grd.alfxold/2.
   grd.dxgas=grd.dxgasold*(exp(grd.alfx)-1)/(exp(grd.alfxold)-1)
   grd.nxgas=2*grd.nxgas
bbb.restart=1
bbb.newgeo=1
bbb.gengrid=1
bbb.isnintp=1
grd.ixdstar = com.nxcore[0,1]+1



# A Python translator that will convert files following a set
# of file-suffix-specific substitution rules
# Written by Ron Cohen, February 2007.  Last revision March 15, 2007.

# import utilities to create filtered list of files 
from filelists import *
import os
import types
import filecmp

"""
create a list of classes to enstantiate which will have conversion rules
for different language files.

Usage:

  Create a file "localrules.py" that defines globalsubrules to apply to
  all file types and any of cppsubrules, pythonsubrules,fortransubrules,
  mpplsubrules,F90subrules that you want to apply to specific file
  types.  These rules are of the form:
  globalsubrules = [("in1","out1"),("in2","out2"),
    ...] where string in1 will be converted to string out1, etc.  It is
    an ordered list which will be executed first to last.
    Another option is that an entry in globalsubrules can be the name
    of a function of one argument (the string being processed).
  Note currently, if the ordered list returns "None" (the python None,
    not the string "None")for a line,
    no further proceessing of that line is done and no line is written.
    This can be used to re-order lines, since a function entry used in
    globalsubrules can be used to return a list of lines, or "None". But
    no further processing is done of the returned list of lines, so any
    such re-ordering must be done as the last of the rules..
  Note in additon to duples, one can also include in the subrules methods
    which take as its sole argument the string the method operates on;
    use this for more complicated operations than simple replacements
  Also create a local script in the directory where you want to
  convert files. Begin that script with "from convert import *",
  making sure that convert.py and localrules.py are in your path.
  Warning: don't end the local script with the suffix .py or the convertor
  will appy your rules to `your local script (assuming you use the default
  suffix as the suffxin for class Py). 

  Further notes

  1.If desired, the default suffixes searched to designate files of
    a specific type can be edited in your local script.  to do so
    import convert, edit the suffix for the sub-class (for example,
    convert.py.suffix = "newsuffix"), then type "from convert import *".

  2.Create sub-classes that inherit from class generic, for any additional
    file suffix that will be converted.  In each subclass, set suffixin
    to a string that gives the suffix of files to be converted, e.g. ".cc"
    or ".py".  If the processed files are to have a different suffix,
    set suffixout to the desired suffix.

  3.Append processall.classlist, a list of sub-class names that will
    be processed to do the conversion, to add the names of any classes
    added in step 2.

  Then, to proces files:
  1.Execute processall(indir,outdir) to process all the specified file
    types.  indir and outdir are optional arguments to specify paths
    for the directory to be processed and the directory where processed
    files will be written.  By default indir = ".", the current directory,
    and outdir = "./converted".
  2.You can also create instances of specific classes created in step 2, and
    just process those files.  For example a = Py(indir,outdir).  See
    documentation for class generic to see available methods.
    WARNING: If you've run the script before and already created the outdir,
    the script will overwrite files in outdir with the same name
  3.The script will only process files if the source file is newer than
    the target (or the target doesn't exist).
  4.You can also use processdirs(dirlist) to run processall in a list of
    directory names specified in dirlist (names in quotes).
"""

# Global substitution dictionary, globalsubdict = {"instring1:outstring1",...}
globalsubdict = {}

# subrules of form [("in1","out1"),("in2","out2"), ...]
globalsubrules = []

# substitution rules to be appended to globalsubrules for each language
cppsubrules = []
# e.g. cppsubrules = [ ("hier::Box<NDIM>",       "Box"),
#    ("hier::IntVector<NDIM>", "IntVect"),
#    ("hier::Index<NDIM>",     "IntVect") ]

pythonsubrules = []

fortransubrules = []

F90subrules = []

F90_90subrules = []

MPPLsubrules = []

try:
  execfile("localrules.py")
except:
  try:
    execfile("../localrules.py")
  except:
    print("No file or problem with 'localrules.py'; proceeding with default rules")

def fnconvert(name,suffixout):
    # Converts a file name "name" by substituting suffixout for the
    # existing suffix, or if there is no suffix, appending suffixout
    suffixin = name.split(".")[-1]
    if (suffixin != name):
        nameout = name.rstrip(suffixin)+suffixout
        # this coding makes sure only last occurence of suffix is repalced
    else:
        nameout = name+"."+suffixout
    return nameout

from stat import ST_MTIME
from numpy import greater
def newer(file0,file1):
    # returns 1 if file0 is newer than file1, 0 otherwise
    # if file0 does not exist, raise a standard python error
    # if file1 does not exist, return 2
    time0 = os.stat(file0)[ST_MTIME]
    try:
        time1 = os.stat(file1)[ST_MTIME]
    except:
        return 2
    return greater(time0,time1)

class processdirs:
    # Process a list of directories to do conversions with processall
    # The default list of directories is ".", the current directory
    # Establishes a set of subdirectories with processed files
    # in a root directory whose default is called "converted" and
    # is parallel to ".".
    def __init__(self,indirs = ["."],outroot = "../converted",clean=None):
        # create the output root directory if it needs to be
        curpath = os.getenv("PWD")
        try:
            os.mkdir(outroot)
            print "Creating directory "+outroot
        except:
            try:
                os.chdir(outroot)
                print "Output directory already exists; proceeding"
                os.chdir(curpath)
            except:
                raise "Can't create output directory"
        for direc in indirs:
            print "Entering directory "+direc
            if direc == ".":
                curdir = curpath.split("/")[-1]
                outdir = outroot+"/"+curdir
            else:
                outdir = outroot + "/"+direc
            processall(direc,outdir,clean)

class processall:
    # Process all files in a directory with rules according to file type
    # as designated by suffix.
    # List of classes of distict file types
    classlist = ["Py","Cpp","Cpp_h","Cpp_hh","Fortran","F90","F90_90","MPPL"]
    def __init__(self,indir=".",outdir="./converted",clean=None):
        for entry in self.classlist:
            a=eval(entry+"("+`indir`+","+`outdir`+",clean="+`clean`+")")
            # Only process file types with non-empty substition rules
            if (a.subrules != []):
                print "processing for file type ",entry
                a.process()

class generic:
    """
generic class for replacing strings in a series of files.
Methods:
   process(): processes the list of files in the directory indir with
     suffix suffixin, and for each one creates a new file in directory
     outdir with the same root name appended by
     the string suffixout (which by default is suffixin).
   processfile(filename): processes a specific file
Notes:
   indir is string with path.  By default it is ".", the current directory
   outdir is by default "./converted".   If the output directory does not
      exist it will be created.  Can be overwritten after instance created
    """

    suffixin = ""
    suffixout = ""
    subrules = globalsubrules
    def __init__(self,indir=".",outdir="./converted",clean=None):
        self.indir = indir
        self.outdir = outdir
        self.clean=clean
        self.doclean = 0   # default is no removal of duplicate files
        if (self.suffixout == ""):
            self.suffixout = self.suffixin
        # create the output directory if it hasn't been created
        try:
            os.mkdir(self.outdir)
        except:
            pass
    def process(self):
        # get the list of files to process
        self.filelist = filesublist(self.indir,self.suffixin)
        print "processing directory",self.indir,"to directory",self.outdir
        # exclude convert.py, localrules.py and filelists.py from filelist
        if (self.suffixin == "py"):
            try:
                self.filelist.remove("convert.py")
            except:
                pass
            try:
                self.filelist.remove("localrules.py")
            except:
                pass
            try:
                self.filelist.remove("filelists.py")
            except:
                pass
        # Set a flag to remove unchanged files if input and output directories are different
        if self.clean and (self.outdir != self.indir):
            # if the input and output directories are different, eliminate files
            # from the output directory that duplicate those in the input directory.
            self.doclean = 1
        else:
            self.doclean = 0
        # Now process files remaining
        for file in self.filelist:
            self.processfile(file)
    def processfile(self,filename):
        # Convert an individual file
        self.infile=self.indir+"/"+filename
        if (self.suffixout == self.suffixin):
            outfilename=filename
        else:
            outfilename=fnconvert(filename,self.suffixout)
        self.outfile = self.outdir+"/"+outfilename
        if (newer(self.infile,self.outfile)):
            # Only process infile if it is newer than outfile
            print "converting file "+self.infile+" to "+self.outfile
            f=open(self.infile,"r")
            g=open(self.outfile,"w")
            lines=f.readlines()
            iline = 0   # just a diagnostic, counts lines
            for line in lines:
              iline = iline + 1
              for rule in self.subrules:
                if (line != None):
                  if type(rule)==types.TupleType or type(rule)==types.ListType:
                    line = string.replace(line,rule[0],rule[1])
                  else:
                    # if it's not a tuple or list, assume it is a method
                    # that executes something
                    # more complicated than a simple replace
                    line = rule(line)
              if (line != None):
                  if (type(line) == types.StringType):
                      g.write(line)
                  if (type(line) == types.ListType):
                      g.writelines(line)
                    
            g.close()
            f.close()
            if self.doclean == 1:
              if filecmp.cmp(self.outfile,self.infile) and self.outfile != self.infile:
                print self.outfile + " is the same as " + self.infile + ", removing"
                os.remove(self.outfile)

class Py(generic):
    suffixin="py"
    subrules = globalsubrules + pythonsubrules

class Cpp(generic):
    suffixin = "C"
    subrules = globalsubrules + cppsubrules

class Cpp_h(Cpp):
    suffixin = "h"

class Cpp_hh(Cpp):
    suffixin = "hh"

class Fortran(generic):
    suffixin = "f"
    subrules = globalsubrules + fortransubrules

class F90(generic):
    suffixin = "F"
    subrules = globalsubrules + F90subrules

class F90_90(F90):
    suffixin = "F90"

class MPPL(generic):
    suffixin = "m"
    subrules = globalsubrules + MPPLsubrules





import numpy as np
import pact.pdb as pdb
from uedge import bbb



def pdb_restore(file):
    """
        Read a pdb file previously written from Uedge. This reads the file and puts
        the 6 standard variables into the correct format.
    """
    try:
        fp = pdb.open(file,'r')
    except:
        print "Couldn't open pdb file ",file
    try:
        bbb.ngs[...]  = np.array(fp.read('ngs@bbb')).reshape(bbb.ngs.shape[::-1]).transpose()
    except:
        print "Couldn't read ngs from  ",file
    try:
        bbb.nis[...]  = np.array(fp.read('nis@bbb')).reshape(bbb.nis.shape[::-1]).transpose()
    except:
        print "Couldn't read nis from  ",file
    try:
        bbb.phis[...]  = np.array(fp.read('phis@bbb')).reshape(bbb.phis.shape[::-1]).transpose()
    except:
        print "Couldn't read phis from  ",file
    try:
        bbb.tes[...]  = np.array(fp.read('tes@bbb')).reshape(bbb.tes.shape[::-1]).transpose()
    except:
        print "Couldn't read tes from  ",file
    try:
        bbb.tis[...]  = np.array(fp.read('tis@bbb')).reshape(bbb.tis.shape[::-1]).transpose()
    except:
        print "Couldn't read tis from  ",file
    try:
        bbb.ups[...]  = np.array(fp.read('ups@bbb')).reshape(bbb.ups.shape[::-1]).transpose()
    except:
        print "Couldn't read ups from  ",file
    try:
        bbb.tgs[...]  = np.array(fp.read('tgs@bbb')).reshape(bbb.tgs.shape[::-1]).transpose()
    except:
        print "Couldn't read tgs from  ",file

    fp.close()






import numpy as np
import netCDF4 as nc
from uedge import bbb



def cdf4_restore(file):
    """
        Read a cdf4 file previously written from Uedge. This reads the file and puts
        the 6 standard variables into the correct format.
    """
    try:
        cf = nc.Dataset(file)
    except:
        print "Couldn't open cdf4 file ",file
        return


    try:
        bbb.ngs[...]  = np.array(cf.variables['ngs'])
    except:
        print "Couldn't read ngs from  ",file
    try:
        bbb.nis[...]  = np.array(cf.variables['nis'])
    except:
        print "Couldn't read nis from  ",file
    try:
        bbb.phis[...]  = np.array(cf.variables['phis'])
    except:
        print "Couldn't read phis from  ",file
    try:
        bbb.tes[...]  = np.array(cf.variables['tes'])
    except:
        print "Couldn't read tes from  ",file
    try:
     bbb.tis[...]  = np.array(cf.variables['tis'])
    except:
        print "Couldn't read tis from  ",file
    try:
        bbb.ups[...]  = np.array(cf.variables['ups'])
    except:
        print "Couldn't read ups from  ",file
    try:
        bbb.tgs[...]  = np.array(cf.variables['tgs'])
    except:
        print "Couldn't read tgs from  ",file

    cf.close()






def cdf4_save(file):
    """
        Write the 6 standard variables into an cdf4 file.
    """
    try:
        hf = h5py.File(file,'w')
        hfg = hf.create_group('bbb')
    except:
        print "Couldn't open cdf4 file ",file
    try:
        hfg.create_dataset('ngs',data=bbb.ngs)
    except:
        print "Couldn't write ngs to  ",file
    try:
        hfg.create_dataset('nis',data=bbb.nis)
    except:
        print "Couldn't write nis to  ",file
    try:
        hfg.create_dataset('phis',data=bbb.phis)
    except:
        print "Couldn't write phis to  ",file
    try:
        hfg.create_dataset('tes',data=bbb.tes)
    except:
        print "Couldn't write tes to  ",file
    try:
        hfg.create_dataset('tis',data=bbb.tis)
    except:
        print "Couldn't write tis to  ",file
    try:
        hfg.create_dataset('ups',data=bbb.ups)
    except:
        print "Couldn't write ups to  ",file
    try:
        hfg.create_dataset('tgs',data=bbb.tgs)
    except:
        print "Couldn't write ups to  ",file

    hf.close()





#rules for converting mppl to f90
# This is generic down to the UEDGE section
import copy
import os
def Use2use(s):
    #Return s if this is a comment
    if (not s[0].isspace()) and s[0] != "U":
        return s
    #Do a substitution if line contains "Use" and it is not
    #part of a comment
    if (s.find("Use")+1):
        if (s.find("!")==-1) or (s.find("Use")<s.find("!")):
            sout=s.replace("("," ")
            sout=sout.replace(")"," ")
            sout=sout.replace("Use","      use")
            return sout
    return s
def Allot(s):
    # converts MPPL allot to F90 allocate
    if (s.find("allot")+1):
        if (s.find("call allot")+1):
            s=s.replace("call allot","allocate")
            s=s.replace('"','')
            s=s.replace("'","")
            s=s.replace(",","(")
            s=s.replace(")","))")
            return s
        s=s.replace(", allot","")
        s=s.replace(",allot","")
    return s
def Nopdb(s):
    if os.getenv('PACT_DIR') == None:
        if (s.find("c!nopdb")+1):
            s=s.replace("c!nopdb","       ")
    return s
def Petsc(s):
    if os.getenv('PETSC_DIR') != None:
        if os.getenv('PARALLEL') == None:
           if (s.find("cunipetsc")+1):
               s=s.replace("cunipetsc","")
        if (s.find("cpetsc")+1):
            s=s.replace("cpetsc","")
    return s

saved_dec=0
in_uses=0
savedlines=[]
def MoveDecs(s):
    global saved_dec,in_uses
    # Return if this is a comment
    if (not s[0].isspace()) and (not saved_dec) and (not in_uses):
        return s
    # collect lines for declarations
    # if we find an "implicit none" statement, store it and remove the line
    sls=s.lstrip().lower()
    indfunc=sls.find("function")
    indcom=sls.find("!")
    functest= (indfunc == -1 or -1<indcom<indfunc)
    # tests to exclude "real function" but allow "real ! function as
    # part of declaration block
    if (sls[0:8]=="implicit") or (sls[0:4]=="real" and functest) \
       or (sls[0:7]=="integer" and functest) \
       or (sls[0:9]=="character") or (sls[0:9]=="parameter") or \
       (sls[0:8]=="external") or (sls[0:9] == "intrinsic") or \
       (sls[0:7]=="logical" and functest) or (sls[0:9]=="dimension") or \
       (sls[0:4] == "data"):
        savedlines.append(s)
        saved_dec=1
        in_uses=0
        return None
    # if we are in the midst of declarations, save also comments (except for
    # "Common block") and continuations and blank lines as part of
    #  what is moved.)
    if (saved_dec==1) and (sls == "" or s[0].lower() == "c" or sls[0]=="!" \
      or s[0]=="*") and (in_uses == 0) \
      and (s.find("Common block")==-1):
        savedlines.append(s)
        return None
    # Check for continuation line in midst of declarations
    if (saved_dec==1) and (len(s)>6):
        if (s[5].isspace() == 0):
            savedlines.append(s)
            return None

    if (sls[0:3] == "use"):
        in_uses=1

    if (saved_dec==1) and (sls != "") and s[0] != "c" and sls[0] != "!" and \
       (sls[0:3] != "use"):
        #This is our first executable statement.  Add it to our saved
        # declarations lines and return them now
        templines = copy.copy(savedlines)
        templines.append(s)
        #empty out savedlines
        savedlines.__delslice__(0,len(savedlines)+1)
        saved_dec = 0
        in_uses=0
        return templines
    return s
        
inelseif = 0
savedelselines=""
def Elseifthen(s):
    # put a "then" at the end of an elseif if it isn't already there
    # need to check to see if next line is a continue
    global inelseif,savedelselines
    # return s if this is a comment
    if (not s[0].isspace()):
        return s
    if s.find("elseif")+1:
        if s.find("then")+1:
            return s
        # set a flag that we are in an "in-else-if" block that needs work
        inelseif = 1
        # If there is no "then" we need to save it to test if the next
        # line is a continuation
        savedelselines = s
        return(None)
    if (inelseif and len(s)>6 and not s[5].isspace()):
        # This is a continue line, add it to savedelselines
        savedelselines += s 
        return(None)
    if (inelseif and (len(s)<6 or s[5].isspace())):
        # No longer in a continue, so process lines
        if savedelselines.find("then")+1:
            savedelselines +=  s
            inelseif = 0
            return savedelselines
        if savedelselines.split("\n")[-2].find("!")+1:
            # if last line in saved lines has a comment,
            #  find index of last comment sign
            last = savedelselines.rfind("!")
            savedelselines=savedelselines[0:last]+ \
                " then "+savedelselines[last:] + s
            inelseif=0
            return savedelselines
        #Otherwise the last line has no comment so insert "then" at end
        savedelselines = savedelselines[0:-1]+" then\n" + s
        inelseif=0
        return savedelselines
    return s

M2Fsubrules = [("#","!"),Use2use,
               ("(Size4)","(kind=4)::"),
               (":: function"," function"),
               (" break "," exit "),
               (" break\n"," exit\n"),
               ("while (","do while ("),
               ("endwhile","end do"),
               ("      call ruthere","c     call ruthere"),
               ("c!include ","#include "),
               Nopdb,
               Petsc,
               Allot,
               Elseifthen,
               MoveDecs
               ]

#-------------------------------------
# Special for UEDGE
wordsizectr=0
def grdproc(s):
    # process to eliminate ifelse write construction
    global wordsizectr
    if (s.find("ifelse([WORDSIZE]")+1):
        s="#if WORDSIZE == 64\n 2001 format(1p3e23.15)\n#else\n 2001 format(1p3d23.15)\n#endif\n"
#        s="#ifndef WORDSIZE\n 2001 format(1p3e23.15)\n#else\n 2001 format(1p3d23.15)\n#endif\n"
        wordsizectr=4
        return s
    elif wordsizectr > 0:
        wordsizectr -= 1
        return None
    else:
        wordsizectr = 0
        return s

M2Fsubrules.insert(6,grdproc)
M2Fsubrules.insert(3,("do i1=","do_i1: do i1="))
M2Fsubrules.insert(4,("break (2) ! exit do_i1","exit do_i1"))
M2Fsubrules.insert(5,("enddo ! do_i1","enddo do_i1"))
M2Fsubrules.insert(6,("scopy","dcopy"))
M2Fsubrules.insert(6,("SCOPY","DCOPY"))
M2Fsubrules.insert(6,("float","real"))
M2Fsubrules.insert(6,("dfloat","real"))





"""
This script contains all the mechanisms required for the Uedge HDF5
conversion/annotation script to function.  The class defined within
(UeConvertVsH5) will take an HDF5 file and annotate it or create a new
file that is vsh5 compliant.
"""

# Import all required libraries
import numpy
import tables

import ueplotdata

class UeConvertVsH5:
    """
    The purpose of this class is to convert a specified uedge file
    into the new vsh5 schema system.  Note that the old file is
    modified instead of a new file being created.  This is because the
    conversion script actually simply adds new annotation to the old
    HDF5 file.  The reasoning for this is for backwards compatibility
    with other scripts that have not yet been converted to read the
    new vizschema.
    """

    def __init__(self, fname=''):
        """
        Constructor, takes no action, unless an input file is
        specified.  If an input file is specified then that file will
        be annotated automatically.  Otherwise files can be annotated
        individually

        *fname: optional filename on which to perform processing
        """

        self.fname = fname
        if ( fname != '' ):
            self.readData(fname)
            self.createUnstructGrid()
            self.annotateFile(fname)

    def readData(self, fname=''):
        """
        This function reads data in from the specified filename.  It
        stores the raw data in preparation for converstion and output
        into the new vsh5 visualization format

        *fname: Optional name of the file to be annotated.  If
         unspecified it is assumed it was set in the constructor
        """
        
        if ( fname == '' ):
            fname = self.fname
        if ( fname == '' ):
            raise ValueError( 'UeConvertVsH5::readData: ' +
                              'file name was unspecified' )

        self.infile = fname
        fh = tables.openFile(fname, 'r')

        # Read the Uedge grid information
        uen = fh.getNode('/bbb/uegrid')
        self.ueg = uen.uegrid.read()
        self.psi = uen.psiVals.read()

        # Read the Uedge variable data
        uen = fh.getNode('/bbb')
        self.nis = uen.nis.read()
        self.ngs = uen.ngs.read()
        self.ups = uen.ups.read()
        self.tes = uen.tes.read()
        self.tis = uen.tis.read()
        self.phis = uen.phis.read()

        fh.close()

    def annotateFile(self, fname=''):
        """
        This function will write data out to the specified filename.
        If no filename is specified then it assumes that data should
        be annotated to the already existing file attached to this
        object.

        *fname: Optional name of the file to be annotated.  If
         unspecified it is assumed it was set previously
        """

        if ( fname == '' ):
            fname = self.fname
        if ( fname == '' ):
            raise ValueError( 'UeConvertVsH5::annotateFile: ' +
                              'file name was unspecified' )
        fh = tables.openFile(fname, 'a')

        # Write the unstructured data to the file
        gn = fh.createGroup( '/', 'unstructGrid', 
                             'Uedge grid annotated for vsh5' )
        fh.setNodeAttr(gn, 'vsType', 'mesh')
        fh.setNodeAttr(gn, 'vsKind', 'unstructured')
        fh.createArray(gn, 'points', self.nodes)
        fh.createArray(gn, 'cells', self.cells)

        nds = []
        # Write all the actual data to the file now
        nds.append( fh.createArray('/', 'nis', self.newNis) )
        nds.append( fh.createArray('/', 'ngs', self.newNgs) )
        nds.append( fh.createArray('/', 'ups', self.newUps) )
        nds.append( fh.createArray('/', 'tes', self.newTes) )
        nds.append( fh.createArray('/', 'tis', self.newTis) )
        nds.append( fh.createArray('/', 'phis', self.newPhis) )
        nds.append( fh.createArray('/', 'psi', self.newPsi) )

        for i in nds:
            fh.setNodeAttr( i, 'vsType',   'variable'     )
            fh.setNodeAttr( i, 'vsMesh',   'unstructGrid' )
            fh.setNodeAttr( i, 'vsCellOffset', 'center'   )

        fh.close()

    def annotateStructured(self, fname=''):
        """
        This function is almost identical to the normal annotateFile
        routine.  However, this one write the data in a 3 part block
        structured format instead of fully unstructured.

        *fname: optional name of the file to be annotated.  If
         unspecified it is assumed that it was set previously
        """

        if ( fname == '' ):
            fname = self.fname
        if ( fname == '' ):
            raise ValueError( 'UeConvertVsH5::annotateStructured: ' +
                              'file name was unspecified' )

        pdata = ueplotdata.UePlotData(self.infile)

        fh = tables.openFile(fname, 'a')

        # Write the structured grid data to the file
        gnr = fh.createGroup( '/', 'structuredData',
                             'Uedge grids annotated for structured vsh5' )

        xx, yy = pdata.getNodalCoreGrid()
        arr = numpy.zeros( (xx.shape[0], xx.shape[1], 2), numpy.float )
        arr[:,:,0] = xx
        arr[:,:,1] = yy
        gn = fh.createGroup( gnr, 'coreGrid',
                             'Uedge core section structured grid' )
        fh.setNodeAttr(gn, 'vsType', 'mesh')
        fh.setNodeAttr(gn, 'vsKind', 'structured')
        fh.createArray(gn, 'points', arr)

        xx, yy = pdata.getNodalSOLGrid()
        arr = numpy.zeros( (xx.shape[0], xx.shape[1], 2), numpy.float )
        arr[:,:,0] = xx
        arr[:,:,1] = yy
        gn = fh.createGroup( gnr, 'solGrid',
                             'Uedge scrape-off layer section grid' )
        fh.setNodeAttr(gn, 'vsType', 'mesh')
        fh.setNodeAttr(gn, 'vsKind', 'structured')
        fh.createArray(gn, 'points', arr)

        xx, yy = pdata.getNodalPFGrid()
        arr = numpy.zeros( (xx.shape[0], xx.shape[1], 2), numpy.float )
        arr[:,:,0] = xx
        arr[:,:,1] = yy
        gn = fh.createGroup( gnr, 'pfGrid',
                             'Uedge private flux layer section grid' )
        fh.setNodeAttr(gn, 'vsType', 'mesh')
        fh.setNodeAttr(gn, 'vsKind', 'structured')
        fh.createArray(gn, 'points', arr)

        nds = []
        ars = ['nis', 'ngs', 'ups', 'tes', 'tis', 'phis']
        for i in ars:
            gn = fh.createArray( gnr, i + 'Core', pdata.getCenteredCoreData(i) )
            fh.setNodeAttr( gn, 'vsType',' variable' )
            fh.setNodeAttr( gn, 'vsMesh', 'coreGrid' )
            fh.setNodeAttr( gn, 'vsCellOffset', 'center' )

            gn = fh.createArray( gnr, i + 'Sol', pdata.getCenteredSOLData(i) )
            fh.setNodeAttr( gn, 'vsType', 'variable' )
            fh.setNodeAttr( gn, 'vsMesh', 'solGrid' )
            fh.setNodeAttr( gn, 'vsCellOffset', 'center' )

            gn = fh.createArray( gnr, i + 'Pf', pdata.getCenteredPFData(i) )
            fh.setNodeAttr( gn, 'vsType', 'variable' )
            fh.setNodeAttr( gn, 'vsMesh', 'pfGrid' )
            fh.setNodeAttr( gn, 'vsCellOffset', 'center' )

        gn = fh.createArray( gnr, 'psiCore', pdata.getCenteredCorePsi() )
        fh.setNodeAttr( gn, 'vsType', 'variable' )
        fh.setNodeAttr( gn, 'vsMesh', 'coreGrid' )
        fh.setNodeAttr( gn, 'vsCellOffset', 'center' )

        gn = fh.createArray( gnr, 'psiSol', pdata.getCenteredSOLPsi() )
        fh.setNodeAttr( gn, 'vsType', 'variable' )
        fh.setNodeAttr( gn, 'vsMesh', 'solGrid' )
        fh.setNodeAttr( gn, 'vsCellOffset', 'center' )

        gn = fh.createArray( gnr, 'psiPf', pdata.getCenteredPFPsi() )
        fh.setNodeAttr( gn, 'vsType', 'variable' )
        fh.setNodeAttr( gn, 'vsMesh', 'pfGrid' )
        fh.setNodeAttr( gn, 'vsCellOffset', 'center' )

        fh.close()

    def createUnstructGrid(self):
        """
        The mapped rectangular Uedge grid gets converted into a
        full-on unstructured grid here.  This is necessary because the
        branch cut mapped rectangular grid of Uedge is incompatible
        with vsh5, however it can be made compliant as an unstruct
        quadrilateral mesh.

        returns: A pair of numpy arrays the first of which is the
        nodelist and the second of which is the unstructured cell list
        """

        nodes = []
        cells = []
        
        for i in range(0, self.psi.shape[0]):
            for j in range(0, self.psi.shape[1]):

                narr = range(0, 5)
                q = 0
                for k in [1, 2, 4, 3]:
                    q = q+1
                    inst = [ self.ueg[i, j, k, 0], self.ueg[i, j, k, 1] ]
                    if ( nodes.count(inst) == 0 ):
                        narr[q] = nodes.__len__()
                        nodes.append(inst)
                    else:
                        narr[q] = nodes.index(inst)

                narr[0] = 4
                cells.append(narr)

        self.nodes = numpy.array(nodes, numpy.float)
        self.cells = numpy.array(cells, numpy.int)

        self.newNis  = numpy.zeros( (self.cells.shape[0], 2), numpy.float )
        self.newNgs  = numpy.zeros( (self.cells.shape[0], 1), numpy.float )
        self.newUps  = numpy.zeros( (self.cells.shape[0], 2), numpy.float )
        self.newTes  = numpy.zeros( (self.cells.shape[0], 1), numpy.float )
        self.newTis  = numpy.zeros( (self.cells.shape[0], 1), numpy.float )
        self.newPhis = numpy.zeros( (self.cells.shape[0], 1), numpy.float )
        self.newPsi  = numpy.zeros( (self.cells.shape[0], 1), numpy.float )
        q = 0
        for i in range(0, self.psi.shape[0]):
            for j in range(0, self.psi.shape[1]):
                self.newNis[q]  = self.nis[i, j]
                self.newNgs[q]  = self.ngs[i, j]
                self.newUps[q]  = self.ups[i, j]
                self.newTes[q]  = self.tes[i, j]
                self.newTis[q]  = self.tis[i, j]
                self.newPhis[q] = self.phis[i, j]
                self.newPsi[q]  = self.psi[i, j, 0]
                q = q+1

        self.nisWMesh  = numpy.zeros( (self.cells.shape[0], 4), numpy.float )
        self.ngsWMesh  = numpy.zeros( (self.cells.shape[0], 3), numpy.float )
        self.upsWMesh  = numpy.zeros( (self.cells.shape[0], 4), numpy.float )
        self.tesWMesh  = numpy.zeros( (self.cells.shape[0], 3), numpy.float )
        self.tisWMesh  = numpy.zeros( (self.cells.shape[0], 3), numpy.float )
        self.phisWMesh = numpy.zeros( (self.cells.shape[0], 3), numpy.float )
        self.psiWMesh  = numpy.zeros( (self.cells.shape[0], 3), numpy.float )
        q = 0;
        for i in range(0, self.psi.shape[0]):
            for j in range(0, self.psi.shape[1]):
                self.nisWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                     self.ueg[i, j, 0, 1],
                                                     self.nis[i, j, 0],
                                                     self.nis[i, j, 1 ] ],
                                                   numpy.float )
                self.ngsWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                     self.ueg[i, j, 0, 1],
                                                     self.ngs[i, j] ],
                                                   numpy.float )
                self.upsWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                     self.ueg[i, j, 0, 1],
                                                     self.ups[i, j, 0],
                                                     self.ups[i, j, 1 ] ],
                                                   numpy.float )
                self.tesWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                     self.ueg[i, j, 0, 1],
                                                     self.tes[i, j] ],
                                                   numpy.float )
                self.tisWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                     self.ueg[i, j, 0, 1],
                                                     self.tis[i, j] ],
                                                   numpy.float )
                self.phisWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                      self.ueg[i, j, 0, 1],
                                                      self.phis[i, j] ],
                                                    numpy.float )
                self.psiWMesh[q, :] = numpy.array( [ self.ueg[i, j, 0, 0],
                                                     self.ueg[i, j, 0, 1],
                                                     self.psi[i, j, 0] ],
                                                   numpy.float )
                q = q+1

        return self.nodes, self.cells



# Interface class for UEDGE quantities needed to communicate with wall model
from uedge import *

joule_per_ev = 1.6022e-19

class UEwall:
  def __init__(self):
# create empty set and dget function dictionaries; will register
# all the set functions into this dict with their proper
# string names.  This will get appended to the corresponding uefacets dictionaries
    self.setMap = {}
    self.getMap = {}

  def initialize(self):
    #as default, turn off switches is that allow setting the neutral flux on divertors and walls 
    #   (default is uewall is inactive)
    bbb.isextpltmod=0
    bbb.isextwallmod=0
    #
    # get limits of the single boundary index for the various surface
    # types (inner lower divertor, inner wall, etc)
    # Note most of these are not really settable but are set by sheath physics;
    #  invoking them will evoke a suitable error message
    bbb.getbdyindexlims()
    # self.setMap["foo"] = 12345.    #test
    self.setMap["density_EW_H2p1"]                 = [self.setNi_EW,1,2,1]
    self.setMap["density_EW_H2"]                   = [self.setNg_EW,1,2]
    self.setMap["rotationFrequency_EW_H2p1"]       = [self.setUp_EW,1,2,1]
    self.setMap["temperature_EW_H2p1"]             = [self.setTi_EW,1,2,1]
    self.setMap["temperature_EW_electron"]         = self.setTe_EW

    self.setMap["ptclFlux_EW_H2p1"]             = [self.setNiFlux_EW,1,2,1]
    self.setMap["ptclFlux_EW_H2"]          = [self.setNgFlux_EW,1,2]
    self.setMap["angularMomentumFlux_EW_H2p1"] = [self.setUpFlux_EW,1,2,1]
    self.setMap["energyFlux_EW_H2p1"]             = [self.setTiFlux_EW,1,2,1]
    self.setMap["energyFlux_EW_electron"]         = self.setTeFlux_EW
    self.setMap["energyFlux_EW_H2"]          = [self.setTgFlux_EW,1,2]

    self.getMap["density_EW_H2"]                   = [self.getNg_EW,1,2]
    self.getMap["density_EW_H2p1"]                 = [self.getNi_EW,1,2,1]
    self.getMap["rotationFrequency_EW_H2p1"]       = [self.getUp_EW,1,2,1]
    self.getMap["temperature_EW_H2"]                   = [self.getTg_EW,1,2]
    self.getMap["temperature_EW_H2p1"]             = [self.getTi_EW,1,2,1]
    self.getMap["temperature_EW_electron"]         = self.getTe_EW

    self.getMap["ptclFlux_EW_H2p1"]             = [self.getNiFlux_EW,1,2,1]
    self.getMap["ptclFlux_EW_H2"]          = [self.getNgFlux_EW,1,2]
    self.getMap["angularMomentumFlux_EW_H2p1"] = [self.getUpFlux_EW,1,2,1]
    self.getMap["energyFlux_EW_H2p1"]             = [self.getTiFlux_EW,1,2,1]
    self.getMap["energyFlux_EW_electron"]         = self.getTeFlux_EW
    self.getMap["energyFlux_EW_H2"]          = [self.getTgFlux_EW,1,2]

  # utility functions to turn on/off external (vs. internal) setting of neutral fluxes at divertors, wall
  def setextneutrals_at_div(self):
     bbb.isextpltmod=1
  def setextneutrals_at_wall(self):
     bbb.isextwallmod=1
  def unsetextneutrals_at_div(self):
     bbb.isextpltmod=0
  def unsetextneutrals_at_wall(self):
     bbb.isextwallmod=0
  def setextneutrals(self):
     setextneutrals_at_div(self)
     setextneutrals_at_wall(self)
  def unsetextneutrals(self):
     unsetextneutrals_at_div(self)
     unsetextneutrals_at_wall(self)

  def setNi_EW(self, val, loc_index,spec_index):
      raise "Ni_EW is computed, can't be set"

  def setNg_EW(self, val, loc_index,spec_index):
      raise "Ng_EW inappropriate variable, set gas flux instead"

  def setUp_EW(self, val, loc_index,spec_index):
    raise "Up_EW is computed, can't be set"
  
  def setTi_EW(self, val, loc_index,spec_index):
    raise "Ti_EW is computed, can't be set"
  
  def setTe_EW(self, val,loc_index):
    raise "Te_EW is computed, can't be set"

  def setTg_EW(self, val,loc_index):
    raise "Tg_EW inappropriate variable, set gas energy flux instead"

  def setNiFlux_EW(self, val, loc_index,spec_index):
    raise "NiFlux_EW is computed, can't be set"

  def setNgFlux_EW(self, val, loc_index,spec_index):
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    # note uedge variable to be set is flux*area
    if wallname == "innerdiv":
      bbb.fngxextlb[iy,spec_index,0] = val*com.sy(ix,iy)
    if wallname == "outerdiv":
      bbb.fngxextrb[iy,spec_index,0] = val*com.sy(ix,iy)
    if wallname == "outerwall":
      bbb.fngyexto[ix,spec_index] = val*com.sx(ix,iy)
    if wallname == "privwall":
      bbb.fngyexti[ix,spec_index] = val*com.sx(ix,iy)

  def setUpFlux_EW(self, val, loc_index,spec_index):
    raise "UpFlux_EW is computed, can't be set"

  def setTiFlux_EW(self, val, loc_index,spec_index):
    raise "TiFlux_EW is computed, can't be set"

  def setTeFlux_EW(self, val,loc_index):
    raise "TeFlux_EW is computed, can't be set"

  def setTgFlux_EW(self, val, loc_index,spec_index):
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    if wallname == "innerdiv":
      print "will set bbb.fegxextlb[iy,spec_index,0] = val*com.sy(ix,iy)"
    if wallname == "outerdiv":
      print "will set bbb.fegxextrb[iy,spec_index,1] = val*com.sy(ix,iy)"
    if wallname == "outerwall":
      print "will set bbb.fegyexto[ix,spec_index,1] = val*com.sx(ix,iy)"
    if wallname == "privwall":
      print "will set bbb.fegyexti[ix,spec_index,1] = val*com.sx(ix,iy)"
    
  def getNi_EW(self, loc_index,spec_index):
    return bbb.get1dpoint(bbb.nis[0:nx+1,0:ny+1,spec_index],loc_index)

  def getNg_EW(self, loc_index,spec_index):
    return bbb.get1dpoint(bbb.ngs[0:nx+1,0:ny+1,spec_index],loc_index)

  def getUp_EW(self, loc_index,spec_index):
    val = bbb.get1dpoint(bbb.ups[0:nx+1,0:ny+1,spec_index],loc_index)
    # convert from uedge's parallel velocity array to toroidal rotation
    # frequency.   
    # convert from uedge's parallel velocity array to toroidal rotation
    rbfbt = bbb.get1dpoint(bbb.rbfbt,loc_index)
    rm = bbb.get1dpoint(com.rm[0:nx+1,0:ny+1,0],loc_index)
    return val*rbfbt/rm

  def getTi_EW(self, loc_index,spec_index):
    return bbb.get1dpoint(bbb.tis[0:nx+1,0:ny+1,spec_index],loc_index)/bbb.qe

  def getTe_EW(self,loc_index):
    return bbb.get1dpoint(bbb.tes,loc_index)/bbb.qe

  def getTg_EW(self, loc_index,spec_index):
    return bbb.get1dpoint(bbb.tg[0:nx+1,0:ny+1,spec_index],loc_index)/bbb.qe

  def getNiFlux_EW(self, loc_index,spec_index):
    ix=0;iy=0;wallname=""
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    if (wallname == "outerwall" or wallname == "privwall"):
      val = bbb.get1dpoint(bbb.fniy[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sy,loc_index)
    if (wallname == "inner_div" or wallname == "outerdiv"):
      val = bbb.get1dpoint(bbb.fnix[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sx,loc_index)

  def getNgFlux_EW(self, loc_index,spec_index):
    ix=0;iy=0;wallname=""
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    if (wallname == "outerwall" or wallname == "privwall"):
      val = bbb.get1dpoint(bbb.fngy[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sy,loc_index)
    if (wallname == "inner_div" or wallname == "outerdiv"):
      val = bbb.get1dpoint(bbb.fngx[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sx,loc_index)

  def getUpFlux_EW(self, loc_index,spec_index):
    ix=0;iy=0;wallname=""
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    if (wallname == "outerwall" or wallname == "privwall"):
      val = bbb.get1dpoint(bbb.fmiy[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sy,loc_index)
    if (wallname == "inner_div" or wallname == "outerdiv"):
      val = bbb.get1dpoint(bbb.fmix[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sx,loc_index)

  def getTiFlux_EW(self, loc_index,spec_index):
    ix=0;iy=0;wallname=""
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    if (wallname == "outerwall" or wallname == "privwall"):
      val = bbb.get1dpoint(bbb.feiy[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sy,loc_index)
    if (wallname == "inner_div" or wallname == "outerdiv"):
      val = bbb.get1dpoint(bbb.feix[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sx,loc_index)

  def getTeFlux_EW(self,loc_index):
    ix=0;iy=0;wallname=""
    bbb.get_ixiybdy(loc_index,ix,iy,wallname)
    if (wallname == "outerwall" or wallname == "privwall"):
      val = bbb.get1dpoint(bbb.feey[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sy,loc_index)
    if (wallname == "inner_div" or wallname == "outerdiv"):
      val = bbb.get1dpoint(bbb.feex[0:nx+1,0:ny+1,spec_index],loc_index)
      return val/bbb.get1dpoint(com.sx,loc_index)

  def getTgFlux_EW(self, loc_index,spec_index):
    # For now the neutral energy flux is just taken as the gas temperature times the
    # neutral particle flux
    return self.getNgFlux_EW(loc_index,spec_index)*self.getTg_EW(loc_index,spec_index)* \
           joule_per_ev
        



#
# @file        uefcifc.py
#
# @brief       Interface for facets interface coupling
#
# @version     $Id: uefcifc.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $
#

# required imports
import uefacets

# This class defines interfaces to Uedge in a fashion that closely
# resembles the FACETS interface in order to facilitate easy coupling
class UedgeFacetsIfc:

    # Constructor.  Functions should get registered in here so as to
    # ensure that the set and get maps are available for the complete
    # lifetime of the UedgeFacetsIfc object.  Internal variables to
    # the ifc class should also be declared here.
    def __init__(self):
        self.ueObj = uefacets.Uedge()

        # create an empty setter function dictionary, and register
        # all the set functions into this dict with their proper
        # string names
        
        self.setMap = {}

        self.setMap["init_file"]    = self.setInFileName
        self.setMap["restore_file"] = self.setRestoreName

        self.setMap["density_CE_H2+1"]                 = self.setNi
        self.setMap["density_CE_neutral"]              = self.setNg
        self.setMap["parallelMomentumDensity_CE_H2+1"] = self.setUp
        self.setMap["temperature_CE_H2+1"]             = self.setTi
        self.setMap["temperature_CE_electron"]         = self.setTe

        # create an empty getter function dictionary and register
        # all the get functions into this dict with their proper
        # string names.
        
        self.getMap = {}

        self.getMap["temperature_CE_electron"]         = self.getTe
        self.getMap["density_CE_neutral"]              = self.getNg
        self.getMap["density_CE_H2+1"]                 = self.getNi
        self.getMap["temperature_CE_H2+1"]             = self.getTi
        self.getMap["parallelMomentumDensity_CE_H2+1"] = self.getUp

    # this constructs the data required for the solver to function
    # no assumptions to the actual solver can be made here, because it
    # is not guaranteed that buildSolver will have been called before
    # this.
    def buildData(self):
        pass

    # this constructs the solver itself.  Assumptions about data can be
    # made, since buildData is guaranteed to be called before this.
    # However, no assumptions can be made about the state of the data, as
    # it is not guaranteed that initialize will have been called before
    # this
    def buildSolver(self):
        pass

    # this initializes the solver and prepares for runs.  This can safely
    # assume that the data has been completely constructed and that the
    # solver is constructed and ready for use.  This is called last in
    # the initialization callpath.
    def initialize(self):
        self.ueObj.readParams(self.infile)
        self.ueObj.restore(self.rsfile)

    # this advances UEDGE to time t
    def advance(self, t):
        self.ueObj.advance(t)

    # this finalizes the UEDGE object.  Note that this call should NOT
    # destroy data.  Instead, all postprocessing of data and solver state
    # should be performed here.  Data destruction should be left up to
    # the class destructor.
    def finalize(self):
        pass

    # set data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map
    def setData(self, name, val):
        self.setMap[name](name, val)

    # get data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map
    def getData(self, name):
        return self.getMap[name](name)

    # set the file name from which to read parameters
    def setInFileName(self, name, fname):
        self.infile = fname

    # set the file name from which to restore
    def setRestoreName(self, name, rname):
        self.rsfile = rname

    # set parameters in the UEDGE object.  Any data processing and
    # massaging that needs to be done should be performed in these
    # functions.
    def setNi(self, name, val):
        n = len(val)
        bbb.isnicore = 1
        bbb.ncore[0:n] = val

    def setNg(self, name, val):
        n = len(val)
        bbb.isngcore = 1
        bbb.ngcore[0:n] = val

    def setUp(self, name, val):
        n = len(val)
        bbb.isupcore = 0
        bbb.upcore[0:n] = val
  
    def setTi(self, name, val):
        n = len(val)
        bbb.iflcore = 0
        bbb.tcorei[0:n] = val * bbb.qe

    def setTe(self, name, val):
        n = len(val)
        bbb.iflcore = 0
        bbb.tcoree[0:n] = val * bbb.qe

    # get parameters from the UEDGE object.  Any data processing and
    # massaging that needs to be done should be performed in these
    # functions
    def getNi(self, name):
        return self.ueObj.getNi()

    def getNg(self, name):
        return self.ueObj.getNg()

    def getUp(self, name):
        return self.ueObj.getUp()

    def getTe(self, name):
        return self.ueObj.getTe()

    def getTi(self, name):
        return self.ueObj.getTi()



#!/usr/bin/env python
# To use: 
#       python setup.py install
#
import sys,os,os.path,string,site
from Forthon.compilers import FCompiler
import getopt

try:
    os.environ['PATH'] += os.pathsep + site.USER_BASE + '/bin'
    import distutils
    from distutils.core import setup, Extension
    from distutils.dist import Distribution
    from distutils.command.build import build
    from subprocess import call
    import numpy
except:
    raise SystemExit, "Distutils problem"

optlist,args = getopt.getopt(sys.argv[1:],'gt:F:',['parallel','petsc'])
machine = sys.platform
debug   = 0
fcomp   = None
parallel = 0
petsc = 0

for o in optlist:
  if   o[0] == '-g': debug = 1
  elif o[0] == '-t': machine = o[1]
  elif o[0] == '-F': fcomp = o[1]
  elif o[0] == '--parallel': parallel = 1
  elif o[0] == '--petsc': petsc = 1  


if petsc == 1 and os.getenv('PETSC_DIR') == None:
    raise SystemExit, "PETSc requested but PETSC_DIR not set"
if  os.getenv('PETSC_DIR') != None:
    petsc = 1
if petsc == 1 and os.getenv('PETSC_ARCH') == None:
    raise SystemExit, "PETSc requested but PETSC_ARCH not set"


sys.argv = ['setup2.py']+args
fcompiler = FCompiler(machine=machine,
                      debug=debug,
                      fcompname=fcomp)
print fcompiler

class uedgeBuild(build):
    def run(self):
        print 'buildit'
        if petsc == 0:
           call(['make','-f','Makefile.Forthon'])
        else:
           call(['make','-f','Makefile.PETSc'])
        build.run(self)
class uedgeClean(build):
    def run(self):
        if petsc == 0:
           call(['make','-f','Makefile.Forthon','clean'])
        else:
           call(['make','-f','Makefile.PETSc','clean'])

#dummydist = Distribution()
#dummybuild = build(dummydist)
#dummybuild.finalize_options()
#builddir = dummybuild.build_temp
builddir = 'build'

uedgepkgs = ['aph','api','bbb','com','flx','grd','svr','wdf']

def makeobjects(pkg):
  return [pkg+'_p.o',pkg+'pymodule.o']

uedgeobjects = []
for pkg in uedgepkgs:
  uedgeobjects = uedgeobjects + makeobjects(pkg)

# add here any extra dot o files other than pkg.o, pkg_p.o


uedgeobjects = uedgeobjects + ['aphrates.o','aphread.o',
                               'apifcn.o','apip93.o','apisorc.o',
                               'fimp.o','fmombal.o','inelrates.o',
                               'sputt.o','boundary.o','convert.o',
                               'geometry.o','griddubl.o','oderhs.o',
                               'odesetup.o','odesolve.o','potencur.o',
                               'ext_neutrals.o',
                               'turbulence.o','blasext.o','brent.o',
                               'comutil.o','misc.o','mnbrak.o',
                               'flxcomp.o','flxdriv.o','flxread.o',
                               'flxwrit.o','grdcomp.o','grddriv.o',
                               'grdinit.o','grdread.o','grdwrit.o',
                               'daspk.o','nksol.o','svrut1.o','svrut2.o',
                               'svrut3.o','svrut4.o','vodpk.o','uoa.o',
                               'dsum.o','dummy_py.o','error.o','getmsg.o',
                               'dmath.o','ssum.o','daux1.o','wdf.o']

if petsc:
  uedgeobjects = uedgeobjects +['petsc-uedge.o','petscMod.o']

if parallel:
  # add extra dot o's needed if we're parallel
  uedgeobjects = uedgeobjects + []

uedgeobjects = map(lambda p:os.path.join(builddir,p),uedgeobjects)

if os.getenv('PACT_DIR') != None:
   library_dirs = fcompiler.libdirs + [
               os.path.join(os.getenv('PACT_DIR'),'lib')]
   libraries = ['pdb','pml','score','blas','m'] + fcompiler.libs 
else:
   library_dirs = fcompiler.libdirs
   libraries = fcompiler.libs 

if petsc:
  #PETSC_DIR = '/homes/mccomic/petsc-uedge'
  #PETSC_ARCH = 'linux-uedge'
  PETSC_DIR=os.getenv('PETSC_DIR')
  PETSC_ARCH=os.getenv('PETSC_ARCH')
  library_dirs = fcompiler.libdirs + [os.path.join(PETSC_DIR,PETSC_ARCH,'lib')]
  libraries =  ['petscts','petscsnes','petscksp','petscdm','petscmat',
                'petscvec','petsc','HYPRE','mpich','lapack','blas','X11',
                'pthread','rt','stdc++','m'] + fcompiler.libs
  libraries =  ['petsc'] + fcompiler.libs

if parallel:
  library_dirs = fcompiler.libdirs + ['/usr/lpp/ppe.poe/lib']
  libraries = fcompiler.libs + ['mpi']
  #uedgeobjects = uedgeobjects + ['/usr/local/mpi/ifc_farg.o']

setup (name = "uedge",
       version = '7.0.7.8',
       author = 'Tom Rognlien',
       author_email = "trognlien@llnl.gov",
       maintainer = 'Bill Meyer',
       maintainer_email = 'meyer8@llnl.gov',
       description = "2D Fluid simulation of plasma and neutrals in magnetic fusion devices",
       platforms = "Unix, Windows (cygwin), Mac OSX",
       packages= ['uedge'],
       package_dir = {'uedge':'pyscripts'},
       #include_package_data=True,
       scripts = ['pyscripts/pdb2hdf5','pyscripts/bas2py'],
       ext_modules = [Extension('uedgeC',
                                ['uedgeC_Forthon.c',
                                 os.path.join(builddir,'Forthon.c'),
                                 'com/handlers.c','com/vector.c'],
                                include_dirs=[builddir,numpy.get_include()],
                                library_dirs=library_dirs,
                                libraries=libraries,
                                define_macros=[("WITH_NUMERIC","0"),
                                               ("FORTHON_PKGNAME", '\"uedgeC\"')],
                                extra_objects=uedgeobjects,
                                extra_link_args=['-g']+
                                             fcompiler.extra_link_args,
                                extra_compile_args=fcompiler.extra_compile_args
                               )],
       cmdclass = { 'build':uedgeBuild,'clean':uedgeClean },
       install_requires=['forthon','mppl']
       ## note that include_dirs may have to be expanded in the line above

       )



# Import all required libraries

import numpy
import tables

#
# The purpose of this class is to read in a UEDGE HDF5 file and then
# make it available in a friendly fashion for plotting
#
class UePlotData:

#
# Constructor
#
# @param fname the file from which this PlotData object should read
#
    def __init__(self, fname):
        self.myName = fname
        self.readData(fname)

#
# This function is called to read data from a given HDF5 file
#
# @param fname the name of the file from which to read
#
    def readData(self, fname, block='tis'):
        fh = tables.openFile(fname)

        # Read the Uedge grid information
        uen = fh.getNode('/bbb/uegrid')
        self.ueg = uen.uegrid.read()
        self.psi = uen.psiVals.read()

        # Read the ion temperature data
        udn = fh.getNode('/bbb')
        self.ued = udn.__getattr__(block).read()

        # Read the additional metadata associated with the branch
        # cut grid
        ifg = fh.getNode('/bbb/uegrid/blockInfo')

        self.ysp  = ifg._v_attrs.__getattr__('ySeparatrix')
        self.xsp1 = ifg._v_attrs.__getattr__('xLeftSeparatrix')
        self.xsp2 = ifg._v_attrs.__getattr__('xRightSeparatrix')

        self.yoff = ifg._v_attrs.__getattr__('yMagneticAxis')
        self.xoff = ifg._v_attrs.__getattr__('xMagneticAxis')

        fh.close()

#
# Returns a tuple of numpy arrays representing the mapped grid of the
# private flux region at the cell nodes.  Grid is +1 on each side since
# all cells are completed
#
# @return a tuple of numpy arrays representing the mapped grid
#     1: The XX array of the mapped grid
#     2: The YY array of the mapped grid
#
    def getNodalPFGrid(self):
        xx = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 1,
                           self.ysp + 1), numpy.float )
        yy = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 1,
                           self.ysp + 1), numpy.float )

        xx[:self.xsp1, :] = self.ueg[1:self.xsp1+1, 1:self.ysp+2, 1, 0]
        xx[self.xsp1:, :] = self.ueg[self.xsp2+1:,  1:self.ysp+2, 1, 0]

        yy[:self.xsp1, :] = self.ueg[1:self.xsp1+1, 1:self.ysp+2, 1, 1] - self.yoff
        yy[self.xsp1:, :] = self.ueg[self.xsp2+1:,  1:self.ysp+2, 1, 1] - self.yoff

        return xx, yy

#
# Returns a tuple of numpy arrays representing the mapped grid of the
# scrape-off region at the cell nodes.  Grid is +1 on each side since
# all cells are completed
#
# @return a tuple of numpy arrays representing the mapped grid
#     1: The XX array of the mapped grid
#     2: The YY array of the mapped grid
#
    def getNodalSOLGrid(self):
        xx = numpy.zeros( (self.ueg.shape[0] - 1,
                           self.ueg.shape[1] - self.ysp - 1), numpy.float )
        yy = numpy.zeros( (self.ueg.shape[0] - 1,
                           self.ueg.shape[1] - self.ysp - 1), numpy.float )

        xx[:, :] = self.ueg[1:, self.ysp+1:, 1, 0]

        yy[:, :] = self.ueg[1:, self.ysp+1:, 1, 1] - self.yoff

        return xx, yy

#
# Returns a tuple of numpy arrays representing the mapped grid of the
# core region at the cell nodes.  Grid is +1 on each side since all
# cells are completed
#
# @return a tuple of numpy arrays representing the mapped grid
#     1: The XX array of the mapped grid
#     2: The YY array of the mapped grid
#
    def getNodalCoreGrid(self):
        xx = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp + 1), numpy.float )
        yy = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp + 1), numpy.float )

        xx[:-1, :] = self.ueg[self.xsp1+1:self.xsp2+1, 1:self.ysp+2, 1, 0]
        xx[-1:, :] = self.ueg[self.xsp1+1,             1:self.ysp+2, 1, 0]

        yy[:-1, :] = self.ueg[self.xsp1+1:self.xsp2+1, 1:self.ysp+2, 1, 1] - self.yoff
        yy[-1:, :] = self.ueg[self.xsp1+1,             1:self.ysp+2, 1, 1] - self.yoff

        return xx, yy

#
# Returns a tuple of numpy arrays representing the mapped grid of the
# private flux region at the cell centers.
#
# @return a tuple of numpy arrays representing the mapped grid
#     1: The XX array of the mapped grid
#     2: The YY array of the mapped grid
#
    def getCenteredPFGrid(self):
        xx = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 2,
                           self.ysp), numpy.float )
        yy = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 2,
                           self.ysp), numpy.float )

        xx[:self.xsp1, :] = self.ueg[1:self.xsp1+1,  1:self.ysp+1, 0, 0]
        xx[self.xsp1:, :] = self.ueg[self.xsp2+1:-1, 1:self.ysp+1, 0, 0]

        yy[:self.xsp1, :] = self.ueg[1:self.xsp1+1,  1:self.ysp+1, 0, 1] - self.yoff
        yy[self.xsp1:, :] = self.ueg[self.xsp2+1:-1, 1:self.ysp+1, 0, 1] - self.yoff

        return xx, yy

#
# Returns a tuple of numpy arrays representing the mapped grid of the
# scrape-off region at the cell centers.
#
# @return a tuple of numpy arrays representing the mapped grid
#     1: The XX array of the mapped grid
#     2: The YY array of the mapped grid
#
    def getCenteredSOLGrid(self):
        xx = numpy.zeros( (self.ueg.shape[0] - 2,
                           self.ueg.shape[1] - self.ysp - 2), numpy.float )
        yy = numpy.zeros( (self.ueg.shape[0] - 2,
                           self.ueg.shape[1] - self.ysp - 2), numpy.float )

        xx[:, :] = self.ueg[1:-1, self.ysp+1:-1, 0, 0]

        yy[:, :] = self.ueg[1:-1, self.ysp+1:-1, 0, 1] - self.yoff

        return xx, yy

#
# Returns a tuple of numpy arrays representing the mapped grid of the
# core region at the cell centers.
#
# @return a tuple of numpy arrays representing the mapped grid
#     1: The XX array of the mapped grid
#     2: The YY array of the mapped grid
#
    def getCenteredCoreGrid(self):
        xx = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp), numpy.float )
        yy = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp), numpy.float )

        xx[:-1, :] = self.ueg[self.xsp1+1:self.xsp2+1, 1:self.ysp+1, 0, 0]
        xx[-1:, :] = self.ueg[self.xsp1+1,             1:self.ysp+1, 0, 0]

        yy[:-1, :] = self.ueg[self.xsp1+1:self.xsp2+1, 1:self.ysp+1, 0, 1] - self.yoff
        yy[-1:, :] = self.ueg[self.xsp1+1,             1:self.ysp+1, 0, 1] - self.yoff

        return xx, yy

#
# Returns a numpy array representing the uedge psi values at the centers
# of every cell in the prival flux region.
#
# @return a numpy array of the psi values
#
    def getCenteredPFPsi(self):
        ps = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 2,
                           self.ysp), numpy.float )

        ps[:self.xsp1, :] = self.psi[1:self.xsp1+1,  1:self.ysp+1, 0]
        ps[self.xsp1:, :] = self.psi[self.xsp2+1:-1, 1:self.ysp+1, 0]

        return ps

#
# Returns a numpy array representing the uedge psi values at the centers
# of every cell in the scrape off layer.
#
# @return a numpy array of the psi values
#
    def getCenteredSOLPsi(self):
        ps = numpy.zeros( (self.ueg.shape[0] - 2,
                           self.ueg.shape[1] - self.ysp - 2), numpy.float )

        ps[:, :] = self.psi[1:-1, self.ysp+1:-1, 0]

        return ps

#
# Returns a numpy array representing the uedge psi values at the centers
# of every cell in the core region.
#
# @return a numpy array of the psi values
#
    def getCenteredCorePsi(self):
        ps = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp), numpy.float )

        ps[:-1, :] = self.psi[self.xsp1+1:self.xsp2+1, 1:self.ysp+1, 0]
        ps[-1:, :] = self.psi[self.xsp1+1,             1:self.ysp+1, 0]

        return ps

#
# Returns a numpy array represeing the uedge data at the cell nodes
# of each grid cell in the private flux region
#
# @return a numpy array of the output data
#
    def getNodalPFData(self):
        da = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 1,
                           self.ysp + 1), numpy.float )

        for i in range(1, self.xsp1+1):
            for j in range(1, self.ysp+2):
                xx = numpy.array( [self.ueg[i-1, j-1, 0, 0],
                                   self.ueg[i  , j-1, 0, 0],
                                   self.ueg[i  , j  , 0, 0],
                                   self.ueg[i-1, j  , 0, 0]] )
                yy = numpy.array( [self.ueg[i-1, j-1, 0, 1],
                                   self.ueg[i  , j-1, 0, 1],
                                   self.ueg[i  , j  , 0, 1],
                                   self.ueg[i-1, j  , 0, 1]] )
                zz = numpy.array( [self.ued[i-1, j-1],
                                   self.ued[i  , j-1],
                                   self.ued[i  , j  ],
                                   self.ued[i-1, j  ]] )

                fn = self.bilinearInterpolate(xx, yy, zz)

                da[i-1, j-1] = fn( self.ueg[i-1, j-1, 1, 0],
                                   self.ueg[i-1, j-1, 1, 1] )

        for i in range(self.xsp2+1, ueg.shape[0]-1):
            for j in range(1, self.ysp+2):
                xx = numpy.array( [self.ueg[i-1, j-1, 0, 0],
                                   self.ueg[i  , j-1, 0, 0],
                                   self.ueg[i  , j  , 0, 0],
                                   self.ueg[i-1, j  , 0, 0]] )
                yy = numpy.array( [self.ueg[i-1, j-1, 0, 1],
                                   self.ueg[i  , j-1, 0, 1],
                                   self.ueg[i  , j  , 0, 1],
                                   self.ueg[i-1, j  , 0, 1]] )
                zz = numpy.array( [self.ued[i-1, j-1],
                                   self.ued[i  , j-1],
                                   self.ued[i  , j  ],
                                   self.ued[i-1, j  ]] )

                fn = self.bilinearInterpolate(xx, yy, zz)

                da[i-1, j-1] = fn( self.ueg[i-1, j-1, 1, 0],
                                   self.ueg[i-1, j-1, 1, 1] )

        return da

#
# Returns a numpy array represeing the uedge data at the cell nodes
# of each grid cell in the scrape-off region
#
# @return a numpy array of the output data
#
    def getNodalSOLData(self):
        da = numpy.zeros( (self.ueg.shape[0] - 2,
                           self.ueg.shape[1] - self.ysp - 2), numpy.float)

        for i in range(1, self.ueg.shape[0]-1):
            for j in range(self.ysp+1, self.ueg.shape[1]-1):
                xx = numpy.array( [self.ueg[i-1, j-1, 0, 0],
                                   self.ueg[i  , j-1, 0, 0],
                                   self.ueg[i  , j  , 0, 0],
                                   self.ueg[i-1, j  , 0, 0]] )
                yy = numpy.array( [self.ueg[i-1, j-1, 0, 1],
                                   self.ueg[i  , j-1, 0, 1],
                                   self.ueg[i  , j  , 0, 1],
                                   self.ueg[i-1, j  , 0, 1]] )
                zz = numpy.array( [self.ued[i-1, j-1],
                                   self.ued[i  , j-1],
                                   self.ued[i  , j  ],
                                   self.ued[i-1, j  ]] )

                fn = self.bilinearInterpolate(xx, yy, zz)

                da[i-1, j-self.ysp-1] = fn( self.ueg[i-1, j-1, 1, 0],
                                            self.ueg[j-1, j-1, 1, 1] )

        return da

#
# Returns a numpy array represeing the uedge data at the cell nodes
# of each grid cell in the core region
#
# @return a numpy array of the output data
#
    def getNodalCoreData(self):
        da = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp + 1), numpy.float )

        for i in range(self.xsp1+1, self.xsp2+1):
            for j in range(1, self.ysp+2):
                xx = numpy.array( [self.ueg[i-1, j-1, 0, 0],
                                   self.ueg[i  , j-1, 0, 0],
                                   self.ueg[i  , j  , 0, 0],
                                   self.ueg[i-1, j  , 0, 0]] )
                yy = numpy.array( [self.ueg[i-1, j-1, 0, 1],
                                   self.ueg[i  , j-1, 0, 1],
                                   self.ueg[i  , j  , 0, 1],
                                   self.ueg[i-1, j  , 0, 1]] )
                zz = numpy.array( [self.ued[i-1, j-1],
                                   self.ued[i  , j-1],
                                   self.ued[i  , j  ],
                                   self.ued[i-1, j  ]] )

                fn = self.bilinearInterpolate(xx, yy, zz)

                da[i-self.xsp1-1, j-1] = fn( self.ueg[i-1, j-1, 1, 0],
                                             self.ueg[i-1, j-1, 1, 1] )

        return da

#
# Returns a numpy array representing the uedge data at the cell centers
# of each grid cell in the private flux region.
#
# @return a numpy array of the output data
#
    def getCenteredPFData(self, block='tis'):
        self.readData(self.myName, block)

        if ( self.ued.shape.__len__() == 3 ):
            cmpts = self.ued.shape[2]
            deep = True
        else:
            cmpts = 1
            deep = False
        da = numpy.zeros( (self.ueg.shape[0] - self.xsp2 + self.xsp1 - 2,
                           self.ysp, cmpts), numpy.float )

        if ( deep ):
            da[:self.xsp1, :] = self.ued[1:self.xsp1+1,  1:self.ysp+1]
            da[self.xsp1:, :] = self.ued[self.xsp2+1:-1, 1:self.ysp+1]
        else:
            da[:self.xsp1, :, 0] = self.ued[1:self.xsp1+1,  1:self.ysp+1]
            da[self.xsp1:, :, 0] = self.ued[self.xsp2+1:-1, 1:self.ysp+1]

        return da

#
# Returns a numpy array representing the uedge data at the cell centers
# of each grid cell in the scrape-off region
#
# @return a numpy array of the output data
#
    def getCenteredSOLData(self, block='tis'):
        self.readData(self.myName, block)

        if ( self.ued.shape.__len__() == 3 ):
            cmpts = self.ued.shape[2]
            deep = True
        else:
            cmpts = 1
            deep = False
        da = numpy.zeros( (self.ueg.shape[0] - 2,
                           self.ueg.shape[1] - self.ysp - 2, cmpts), numpy.float )

        if ( deep ):
            da[:, :] = self.ued[1:-1, self.ysp+2:]
        else:
            da[:, :, 0] = self.ued[1:-1, self.ysp+2:]

        return da

#
# Returns a numpy array representing the uedge data at the cell centers
# of each grid cell in the core region
#
# @return a numpy array of the output data
#
    def getCenteredCoreData(self, block='tis'):
        self.readData(self.myName, block)

        if ( self.ued.shape.__len__() == 3 ):
            cmpts = self.ued.shape[2]
            deep = True
        else:
            cmpts = 1
            deep = False
        da = numpy.zeros( (self.xsp2 - self.xsp1 + 1,
                           self.ysp, cmpts), numpy.float )

        if ( deep ):
            da[:-1, :] = self.ued[self.xsp1+1:self.xsp2+1, 1:self.ysp+1]
            da[-1:, :] = self.ued[self.xsp1+1,             1:self.ysp+1]
        else:
            da[:-1, :, 0] = self.ued[self.xsp1+1:self.xsp2+1, 1:self.ysp+1]
            da[-1:, :, 0] = self.ued[self.xsp1+1,             1:self.ysp+1]

        return da

#
# Returns a lambda function that can be used to query the value of a
# bilinear interpolation given 4 coordinates and the value at each
# coordinate
#
# @param xx x coordinates of the points to be interpolated
# @param yy y coordinates of the points to be interpolated
# @param zz z values of the points to be interpolated
#
# @return a lambda function representing the interpolation
#
    def bilinearInterpolate(self, xx, yy, zz):

        # These coefficients were auto-generated by Mathematica
        a = (-(xx[0]*xx[3]*(yy[0] - yy[3])*(yy[2]*zz[1] - yy[1]*zz[2])) + xx[2]*(-(xx[3]*(yy[2] - yy[3])*(yy[1]*zz[0]    \
            - yy[0]*zz[1])) + xx[0]*(yy[0] - yy[2])*(yy[3]*zz[1] - yy[1]*zz[3])) + xx[1]*(xx[3]*(yy[1] - yy[3])          \
            * (yy[2]*zz[0] - yy[0]*zz[2]) - xx[2]*(yy[1] - yy[2])*(yy[3]*zz[0] - yy[0]*zz[3]) - xx[0]*(yy[0] - yy[1])    \
            * (yy[3]*zz[2] - yy[2]*zz[3])))/(xx[1]*(xx[2]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[3]*(yy[0] - yy[2])*(yy[1] \
            - yy[3])) + xx[0]*(xx[3]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[2]*(yy[0] - yy[2])*(yy[1] - yy[3])             \
            + xx[1]*(yy[0] - yy[1])*(yy[2] - yy[3])) + xx[2]*xx[3]*(yy[0] - yy[1])*(yy[2] - yy[3]))
        b = (xx[3]*yy[3]*(yy[2]*(zz[0] - zz[1]) + yy[0]*(zz[1] - zz[2]) + yy[1]*(-zz[0] + zz[2])) + xx[1]*yy[1]*(yy[3]   \
            * (zz[0] - zz[2]) + yy[0]*(zz[2] - zz[3]) + yy[2]*(-zz[0] + zz[3])) + xx[2]*yy[2]*(yy[3]*(-zz[0] + zz[1])    \
            + yy[1]*(zz[0] - zz[3]) + yy[0]*(-zz[1] + zz[3])) + xx[0]*yy[0]*(yy[3]*(-zz[1] + zz[2]) + yy[2]*(zz[1]       \
            - zz[3]) + yy[1]*(-zz[2] + zz[3])))/(xx[1]*(xx[2]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[3]*(yy[0] - yy[2])    \
            * (yy[1] - yy[3])) + xx[0]*(xx[3]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[2]*(yy[0] - yy[2])*(yy[1] - yy[3])    \
            + xx[1]*(yy[0] - yy[1])*(yy[2] - yy[3])) + xx[2]*xx[3]*(yy[0] - yy[1])*(yy[2] - yy[3]))
        c = (xx[0]*xx[3]*(yy[0] - yy[3])*(zz[1] - zz[2]) + xx[2]*(xx[3]*(yy[2] - yy[3])*(zz[0] - zz[1]) - xx[0]*(yy[0]   \
            - yy[2])*(zz[1] - zz[3])) + xx[1]*(-(xx[3]*(yy[1] - yy[3])*(zz[0] - zz[2])) + xx[2]*(yy[1] - yy[2])          \
            * (zz[0] - zz[3]) + xx[0]*(yy[0] - yy[1])*(zz[2] - zz[3])))/(xx[1]*(xx[2]*(yy[1] - yy[2])*(yy[0] - yy[3])    \
            - xx[3]*(yy[0] - yy[2])*(yy[1] - yy[3])) + xx[0]*(xx[3]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[2]*(yy[0]       \
            - yy[2])*(yy[1] - yy[3]) + xx[1]*(yy[0] - yy[1])*(yy[2] - yy[3])) + xx[2]*xx[3]*(yy[0] - yy[1])*(yy[2]       \
            - yy[3]))
        d = (xx[3]*(yy[2]*(-zz[0] + zz[1]) + yy[1]*(zz[0] - zz[2]) + yy[0]*(-zz[1] + zz[2])) + xx[2]*(yy[3]*(zz[0]       \
            - zz[1]) + yy[0]*(zz[1] - zz[3]) + yy[1]*(-zz[0] + zz[3])) + xx[0]*(yy[3]*(zz[1] - zz[2]) + yy[1]*(zz[2]     \
            - zz[3]) + yy[2]*(-zz[1] + zz[3])) + xx[1]*(yy[3]*(-zz[0] + zz[2]) + yy[2]*(zz[0] - zz[3]) + yy[0]           \
            * (-zz[2] + zz[3])))/(xx[1]*(xx[2]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[3]*(yy[0] - yy[2])*(yy[1]            \
            - yy[3])) + xx[0]*(xx[3]*(yy[1] - yy[2])*(yy[0] - yy[3]) - xx[2]*(yy[0] - yy[2])*(yy[1] - yy[3]) + xx[1]     \
            * (yy[0] - yy[1])*(yy[2] - yy[3])) + xx[2]*xx[3]*(yy[0] - yy[1])*(yy[2] - yy[3]))

        fn = lambda x, y: a + b*x + c*y + d*x*y

        return fn



# This version hand-processed following bas2py conversion
# This file runs a time-dependent case using dtreal.  First, read rdinitdt
# to set variables, and: read this file.
# If a mistake is made, to restart this file without a Jacobian evaluation,
# be sure to reset iterm=1 (=> last step was successful)

from uedge import *
import PW
          
# INITIALIZE PARAMS -- SHOULD BE DONE IN MASTER SCRIPT OR TERMINAL SESSION
# BEFORE INVOKING THIS SCRIPT
#execfile("rdinitdt.py")
no = 0;yes=1
echo = no

# allocate space -- SHOULD BE DONE IN MASTER SCRIPT OR TERMINAL SSESSION
# BEFORE INVOKING THIS SCRIPT
# bbb.allocate()

nx=com.nx;ny=com.ny;nisp=com.nisp;ngsp=com.ngsp;numvar=bbb.numvar
isteon=bbb.isteon
if (i_stor==0):
   ni_stor = zeros((n_stor,nx+1+1,ny+1+1,nisp),"d")	#set storage arrays
   up_stor = zeros((n_stor,nx+1+1,ny+1+1,nisp),"d")
   te_stor = zeros((n_stor,nx+1+1,ny+1+1),"d")
   ti_stor = zeros((n_stor,nx+1+1,ny+1+1),"d")
   ng_stor = zeros((n_stor,nx+1+1,ny+1+1,ngsp),"d")
   phi_stor = zeros((n_stor,nx+1+1,ny+1+1),"d")
   tim_stor = zeros((n_stor),"d")
   dtreal_stor = zeros((n_stor),"d")
   nfe_stor = zeros((n_stor),"l")
   dt_stor = (tstor_e - tstor_s)/(n_stor - 1)
   # character*80 s(1)
   sfn ="fs=PW.PW('pfdt_"+savefname+"' ,'w')"
   # character*80 s2(1)
   if (n_stor>0): s2 ="fs2=PW.PW(pftstor_"+savefname+" ,'w')"
i_stor = max(i_stor,1)			# set counter for storage arrays
dt_tot = max(dt_tot,0.)
nfe_tot = max(nfe_tot,0)
deldt_0 = bbb.deldt
# real8 fnrm_old
isdtsf_sav = bbb.isdtsfscal
if (ipt==1 and isteon==1): 	#reset ipt to te(nx,iysptrx+1) if no user value
   ipt = bbb.idxte[nx-1,com.iysptrx]
#
##exmain
irev = -1         # forces second branch of irev loop below
if (bbb.iterm == 1):  # successful initial run with dtreal
   bbb.dtreal = bbb.dtreal/mult_dt     # causes same dt after irev loop
else:             # unsuccessful initial run with dtreal; reduce dtreal
   bbb.dtreal = bbb.dtreal/(3*mult_dt) # causes dt=dt/mult_dt after irev loop
   
if (initjac == 0): bbb.newgeo=0
dtreal_sav = bbb.dtreal
bbb.itermx = 10
##ftol = ftol_dt 
##bbb.dtreal = bbb.dtreal/mult_dt		#adjust dtreal for multiplic. to follow
bbb.dtphi = rdtphidtr*bbb.dtreal
irev = -1
# allocate space -- is this really where we should do it?
#bbb.allocate()
#introduce some shortcut na mes
##yl = bbb.yl
##yldot=bbb.yldot
neq=bbb.neq
svrpkg=bbb.svrpkg.tostring().strip()
#
bbb.ylodt = bbb.yl
bbb.pandf1 (-1, -1, 0, bbb.neq, 1., bbb.yl, bbb.yldot)
fnrm_old = sqrt(sum((bbb.yldot[0:neq]*bbb.sfscal[0:neq])**2))
if (initjac == 1): fnrm_old=1.e20
print "initial fnrm =",fnrm_old

for ii1 in range( 1, ii1max+1):
   if (ismfnkauto==1): bbb.mfnksol = 3
   #set the time-step
   if (irev == 0):
      # Only used after a dt reduc. success. completes loop ii2 for fixed dt
      bbb.dtreal = min(3*bbb.dtreal,t_stop)	#first move forward after reduction
      bbb.dtphi = rdtphidtr*bbb.dtreal
      if (ismfnkauto==1 and bbb.dtreal > dtmfnk3): bbb.mfnksol = -3
      bbb.deldt =  3*bbb.deldt
      print ' increase dtreal to ',bbb.dtreal
   else:
      # Most common branch for either increasing to decreasing dtreal
      bbb.dtreal = min(mult_dt*bbb.dtreal,t_stop)
      bbb.dtphi = rdtphidtr*bbb.dtreal
      if (ismfnkauto==1 and bbb.dtreal > dtmfnk3): bbb.mfnksol = -3
      bbb.deldt =  mult_dt*bbb.deldt
      print ' decrease dtreal to ',bbb.dtreal,' by 1/mult_dt = ',1./mult_dt
   bbb.dtreal = min(bbb.dtreal,dt_max)
   bbb.dtphi = rdtphidtr*bbb.dtreal
   if (ismfnkauto==1 and bbb.dtreal > dtmfnk3): bbb.mfnksol = -3
   bbb.deldt = min(bbb.deldt,deldt_0)
   bbb.deldt = max(bbb.deldt,deldt_min)
   nsteps_nk=1
   print ' '
   print '*** Number time-step changes and dt: idtc; bbb.dtreal = ', ii1,';', bbb.dtreal
   print '------------------------------------------------------------------------------'
   ##   ftol = ftol_dt/(1 + 100*bbb.dtreal/dt_max) # gives better converg at large dt
   bbb.itermx = 10
   if (ii1>1  or  initjac==1):	# first time calc Jac if initjac=1
      bbb.icntnunk = 0
      bbb.isdtsfscal = isdtsf_sav
      bbb.ftol = min(ftol_dt, 0.01*fnrm_old)
      exmain() # take a single step at the present bbb.dtreal
      if (bbb.iterm == 1):
         dt_tot = dt_tot + bbb.dtreal
         nfe_tot = nfe_tot + bbb.nfe[0,0]
         bbb.ylodt = bbb.yl
         bbb.pandf1 (-1, -1, 0, bbb.neq, 1., bbb.yl, bbb.yldot)
         fnrm_old = sqrt(sum((bbb.yldot[0:neq-1]*bbb.sfscal[0:neq-1])**2))
         if (dt_tot>=0.999999999999*t_stop  or  fnrm_old<ftol_min):
            print ' total time this advance: dt_tot = ', dt_tot
            print ' '
            print '*****************************************************'
            print '**  SUCCESS: frnm < bbb.ftol; or dt_tot >= t_stop  **'
            print '*****************************************************'
            break

   #bbb.icntnunk=1 #turn off the automatic recalculation of initial Jacobians for subsequent steps
   bbb.isdtsfscal = 0
   for ii2 in range( 1, ii2max+1): #take ii2max steps at the present time-step
      if (bbb.iterm == 1):
         bbb.itermx = 10
         bbb.ftol = min(ftol_dt, 0.01*fnrm_old)
         bbb.exmain()
         if (bbb.iterm == 1):
            bbb.ylodt = bbb.yl
            bbb.pandf1 (-1, -1, 0, bbb.neq, 1., bbb.yl, bbb.yldot)
            fnrm_old = sqrt(sum((bbb.yldot[0:neq-1]*bbb.sfscal[0:neq-1])**2))
            print "* Step number at this dtreal:",ii2,"; initial fnrm =",fnrm_old
            print 'Number of func eval',bbb.nfe[0,0],'; nonlinear iters',bbb.nni[0,0],'; preJac eval = ',bbb.npe[0,0]
            dtreal_sav = bbb.dtreal
            dt_tot = dt_tot + bbb.dtreal
            nfe_tot = nfe_tot + bbb.nfe[0,0]
            if (dt_tot>=0.999999999999*t_stop  or  fnrm_old<ftol_min):
               print ' total time this advance: dt_tot = ', dt_tot
               print ' '
               print '*****************************************************'
               print '**  SUCCESS: frnm < bbb.ftol; or dt_tot >= t_stop  **'
               print '*****************************************************'
               break
            print "Total time = ",dt_tot,"; Timestep = ",bbb.dtreal,"; bbb.yl[ipt], ipt = ",bbb.yl[ipt],", ",ipt
            print " "
##       Store variables if a storage time has been crossed
            if (dt_tot >= dt_stor*i_stor and i_stor<=n_stor):
               i_stor1 = i_stor-1
               ni_stor[i_stor1,:,:,:] = ni
               up_stor[i_stor1,:,:,:] = up
               te_stor[i_stor1,:,:] = te
               ti_stor1[i_stor1,:,:] = ti
               ng_stor[i_stor1,:,:,:] = ng
               phi_stor1[i_stor1,:,:] = phi
               tim_stor[i_stor1] = dt_tot
               nfe_stor[i_stor1] = nfe_tot
               dtreal_stor[i_stor1] = bbb.dtreal
               i_stor = i_stor + 1
   ##          End of storage section
## Write out the time-dependent arrays in outer loop if i_stor > 1
                  
   exec(sfn) #see file rdinitdt - this creates pfdt_"+savefname+"
   fs.nis=bbb.nis
   fs.ups=bbb.ups
   fs.tes=bbb.tes
   fs.tis=bbb.tis
   fs.ngs=bbb.ngs
   fs.phis=bbb.phis
   fs.dtreal_sav=dtreal_sav
   fs.dt_tot=dt_tot
   fs.ii1=ii1
   fs.close()
   if (i_stor > 1):
      exec(s2)
      fs2.ni_stor=ni_stor;fs2.up_stor=up_stor;fs2.te_stor=te_stor
      fs2.ti_stor=ti_stor;fs2.ng_stor=ng_stor;fs2.phi_stor=phi_stor
      fs2.tim_stor=tim_stor
      fs2.close()
      
   if (dt_tot>=t_stop  or  fnrm_old<ftol_min): break   # need for both loops
   irev = irev-1
   if (bbb.iterm != 1):	#print bad eqn, cut dtreal by 3, set irev flag
            ####### just a copy of idtroub script ########################
      oldecho=echo
      echo=no
      # integer ii
      # real8 ydmax 
      scalfac = bbb.sfscal
      if (svrpkg != "nksol"): scalfac = 1/(bbb.yl + 1.e-30)  # for time-dep calc.
      ydmax = 0.999999999*max(abs(bbb.yldot*scalfac))
      itrouble = 0
      for ii in range(neq):
         if (abs(bbb.yldot[ii]*scalfac[ii]) > ydmax):
            itrouble=ii
            print "** Fortran index of trouble making equation is:"
            print itrouble+1
            break
      print "** Number of variables is:"
      print "numvar = ", numvar
      print " "
      iv_t = (itrouble).__mod__(numvar) + 1
      print "** Troublemaker equation is:"
      print "iv_t = ",iv_t
      print " "
      print "** Troublemaker cell (ix,iy) is:"
      print bbb.igyl[itrouble,]
      print " "
      print "** Timestep for troublemaker equation:"
      print bbb.dtuse[itrouble]
      print " "
      print "** yl for troublemaker equation:"
      print bbb.yl[itrouble]
      print " "
      echo=oldecho
      ######## end of idtroub script ##############################

      irev = 1
      print '*** Converg. fails for bbb.dtreal; reduce time-step by 3, try again'
      print '----------------------------------------------------------------- '
      bbb.dtreal = bbb.dtreal/(3*mult_dt)
      bbb.dtphi = rdtphidtr*bbb.dtreal
      if (ismfnkauto==1 and bbb.dtreal > dtmfnk3): bbb.mfnksol = -3
      bbb.deldt =  bbb.deldt/(3*mult_dt) 
      bbb.iterm = 1
echo = yes
